# HYPE (HYbrid Population balance Equation)

## Motivation

Several physical problems involve the evolution of non-inertial solid particles of different sizes evolving in a fluid phase. For example the study of soot formation or crystallizers involves simultaneous nucleation, surface growth/loss and agglomeration (sometimes breakage). In such problems, the Particle Size Distribution (PSD) gives statistical information on particles sizes. This quantity is of interest to describe the evolution of the aerosol. The PSD evolution is modeled by the Population Balance Equation (PBE). This equation has no analytical solution in general. Several numerical methods exist for its resolution, each with its advantages and drawbacks. A new hybrid stochastic/fixed-sectional method was proposed in [Bouaniche et al, *Chemical Engineering Science*, 2019](https://www.sciencedirect.com/science/article/pii/S0009250919306906). This method, called HYPE (for HYbrid Population balance Equation) was specifically designed to avoid the numerical diffusion linked to the resolution of particle growth with sectional methods, keeping a relatively low CPU cost as compared to full Monte Carlo methods. This repository contains the implementation of the HYPE method.

## Repository
The repository is organized as follows: all the source code is located in the `soot_stochastic` folder. In `samples/tutos` typical use cases are provided. Corresponding tutorials are available in the wiki (see next paragraph).

## Tutorials

Tutorials are available to walk you through typical test cases. It is preferable to go through them in the following order:

1. [Agglomeration and growth](https://gitlab.coria-cfd.fr/bouaniche/nps/wikis/tutorials/analytical/agglo-and-growth)
2. [Cantera premixed flame](https://gitlab.coria-cfd.fr/bouaniche/nps/wikis/cantera%20premixed%20flame)
3. [Soot particle size distribution calculation](https://gitlab.coria-cfd.fr/bouaniche/nps/wikis/soot%20particle%20size%20distribution%20calculation)

## Additional documentation

[User inputs](https://gitlab.coria-cfd.fr/bouaniche/nps/wikis/user-inputs): Description of the main choices that can be made in the `main.cpp` file for the configuration of new runs.

## Illustration

![soot_clusters](/uploads/ede715a1e444f35403bb2793eb5ceb3f/soot_clusters.gif)
