library(readr)  # for read_csv()
library(dplyr)  # for mutate()
library(tidyr)  # for unnest()
library(purrr)  # for map(), reduce()

library(ggplot2)
library(gganimate)
# from https://github.com/thomasp85/gganimate


# find all file names ending in .csv 
#files <- dir(pattern = "*.csv")
#data_path <- "/Users/bouaniche/nps/runs/animations/jaerosci/rigopoulos1/hybrid/Np_1e4/outputs/psd/"   # path to the data
data_path <- getwd()
files <- dir(data_path, pattern = "*.dat") # get file names

#clusters_path <- "/Users/bouaniche/nps/runs/animations/jaerosci/rigopoulos1/hybrid/Np_1e4/outputs/psd/psdClusters/"
clusters_path <- paste(data_path, "/psdClusters/", sep = "")
clusters_files <- dir(clusters_path, pattern = "*.dat")

#data <- files %>%
#  map(read.table) %>%
#  reduce(rbind)        # reduce with rbind into one dataframe
#data

psd_t <- files %>%
  # read in all the files, appending the path before the filename
  map(~ read.table(file.path(data_path, .))) %>% 
  reduce(rbind)

psd_t <- as_tibble(psd_t)

colnames(psd_t) <- c("mAvg", "np", "Nv", "ndm", "ndmStoch", "NvR", "deltam", 
                      "mInf", "mSup", "mAvgVar", "mAvgVarStoch", "mAvgVarR", "t")

psd_clusters <- clusters_files %>%
  map(~ read.table(file.path(clusters_path, .))) %>% 
  reduce(rbind)

psd_clusters <- as_tibble(psd_clusters)

colnames(psd_clusters) <- c("mAvgVar", "ndm", "deltam", "t")

psd_clusters <- psd_clusters %>%
  mutate(Nv = ndm*deltam, dataType = rep("clusters", nrow(psd_clusters)))

psd_clusters_selected <- psd_clusters %>%
  select(mAvgVar, Nv, ndm, t, dataType)

psd_t <- psd_t %>%
  mutate(dataType = rep("sections",nrow(psd_t)))

psd_t_selected <- psd_t %>%
  select(mAvgVar, Nv, ndm, t, dataType)

#psd_t_selected <- psd_t %>%
#  select(mAvgVar, Nv, ndm, t, dataType, np)

psd_all <- bind_rows(psd_t_selected, psd_clusters_selected)
#psd_all <- psd_t_selected

# ggplot(psd_t, aes(mAvgVar, ndm)) +
#   geom_point(alpha = 0.7, show.legend = FALSE) +
#   scale_x_log10() +
#   scale_y_log10(limits = c(1e-5, 1)) +
#   scale_fill_brewer(palette = 1) +
#   theme_bw() +
#  # Here comes the gganimate specific bits
#   labs(title = 'Time: {round(frame_time,1)}', x = 'v', y = 'n(v)') +
#   transition_time(t) +
#   ease_aes('linear')

ggplot(psd_all, aes(mAvgVar, ndm * mAvgVar, fill = dataType, shape = dataType)) +
  geom_point(alpha = 1.0, show.legend = TRUE, aes(colour = dataType)) +
  #scale_x_log10() +
  scale_x_log10(limits = c(5e-4, 5e5)) +
  #scale_x_log10(limits = c(2, 1e5)) +
  #scale_x_continuous(limits = c(5, 30)) +
  #scale_y_log10(limits = c(1e-4, 2e-2)) +
  scale_y_log10(limits = c(1e-4, 1)) +
  #scale_y_continuous(limits = c(0.05, 0.2)) +
  scale_color_manual(values=c("blue", "black")) +
  theme_bw() +
  # Here comes the gganimate specific bits
  labs(title = 'Time: {round(frame_time,1)}', x = 'v', y = 'n(v)v') +
  transition_time(t) +
  ease_aes('linear') +
  theme(axis.text.x = element_text(size = 20),
        axis.text.y = element_text(size = 20),
        axis.title.x = element_text(size = 20),
        axis.title.y = element_text(size = 20),
        legend.text = element_text(size = 20),
        legend.title = element_text(size = 20),
        legend.position = 'bottom',
        panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(),
        panel.background = element_blank(), 
        axis.line = element_line(colour = "black"))


# save
anim_name <- "clusters.gif"
anim_path <- data_path
anim_save(filename = anim_name, path = anim_path)
