library(tidyverse)

flame <- read_csv("c2h4_flame_fixed_T.csv")

# add u_i^sup = (u_i + u_{i+1} ) / 2 . Then delta_z = z_{i+1} - z_i
# delta_z / u_i^sup = delta_t^sup. then t_0 = 0 and t_i = t_{i-1} + delta_t_{i-1}^sup

# technically, need to add a new column u_{i+1}

# copy the values of u (and z) excluding the first one
u_i_plus_one <- flame$`u (m/s)`[2: nrow(flame)]
z_i_plus_one <- flame$`z (m)`[2: nrow(flame)]

# extrapolate adding one value equal to the last value of u
u_i_plus_one[nrow(flame)] <- u_i_plus_one[nrow(flame)-1]
z_i_plus_one[nrow(flame)] <- z_i_plus_one[nrow(flame)-1]

# add u_i_plus_one to the data frame
f <- flame %>%
  mutate(u_i_plus_one = u_i_plus_one) %>%
  mutate(z_i_plus_one = z_i_plus_one)

f <- f %>%
  mutate(u_i_sup = 0.5*(`u (m/s)` + u_i_plus_one) ) %>%
  mutate(delta_z = z_i_plus_one - `z (m)`) %>%
  mutate(delta_t = delta_z / u_i_sup)

calc_t <- function(i, f){
  if (i == 1) {
    t = 0
  } else {
    delta_t_vector <- f$delta_t[1:(i-1)]
    t = sum(delta_t_vector)
  }
  return(t)
}

t <- sapply(1:nrow(f), function(i) calc_t(i,f))

# PAHnucl is defined as equal to A4. YA4imposed is not used as we keep 
# string PAHType = "corrected" and  A4correctionType = "PAHsources";
# options not used like m_PAHType == "hybrid" should be deleted.
# for now put 0 in YA4imposed
f <- f %>%
  mutate(t = t) %>%
  mutate(zeros = rep(0, nrow(f))) %>%
  mutate(PAHnucl = A4) %>%
  mutate(YA4imposed = rep(0, nrow(f))) %>%
  mutate(CmA4 = A4*`rho (kg/m3)`/1000)

# mutate CmA4_i_plus_one in order to be able to calculate wmA4
CmA4_i_plus_one <- f$CmA4[2: nrow(f)]
CmA4_i_plus_one[nrow(f)] <- CmA4_i_plus_one[nrow(f)-1]

f <- f %>%
  mutate(CmA4_i_plus_one = CmA4_i_plus_one) %>%
  mutate(wmA4 = (.data$CmA4_i_plus_one - .data$CmA4) / .data$delta_t)

# delta_t = 0 for the last cell so we impose wmA4 of last cell equal to last - 1
f$wmA4[nrow(f)] <- f$wmA4[nrow(f)-1]

# wmA4 is the chemical source term from cantera output in [g/cm3/s] 
# first CmA4 must be calculated (massic concentration) in [g/cm3]
# CmA4 = YA4 * rho [g/dm3] / 1000  -> [g/cm3]
# wmA4 = (CmA4_i_plus_one - CmA4_i) / delta_t  [g/cm3/s]
f_select <- f %>%
  select(zeros, `z (m)`, t, `T (K)`, `u (m/s)`, CO, CO2, C2H4, O2, H2O, H2, A1,
         A2, A3, A4, C2H2, OH, H, PAHnucl, `rho (kg/m3)`, YA4imposed, wmA4)

# filter t < 0  -> no occurence
colnames_tbl <- f_select %>%
  filter(t < 0)

# write output
write.table(f_select, file = "data_parsed/R_data.txt", col.names = F, row.names = F, sep="\t")
write.table(colnames_tbl, file = "data_parsed/R_labels.txt", row.names = F, sep="\t")