import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

path = "./"

sol = pd.read_csv(path + "c2h4_flame_fixed_T.csv")

z = sol["z (m)"]

# Plot main species
plt.figure(figsize = (5,5))

# conversion from [m] to [cm]
z_cm = 100 * z

# plot solved data
plt.plot(z_cm, sol["CO2"], 'k', label='CO2')
plt.plot(z_cm, sol["CO"], 'k:', label='CO')
plt.plot(z_cm, sol["O2"], 'k--', label='O2')
plt.plot(z_cm, sol["H2O"], 'k-.', label='HO2')
plt.plot(z_cm, sol["C2H4"], 'k', label='C2H4')
plt.plot(z_cm, sol["H2"], 'k', label='H2')

# Load ref measured data from paper

xC2H4, YC2H4ref = np.loadtxt(path + "exp_Zhao/" + 'YC2H4.csv', delimiter=';', unpack=True)
xCO, YCOref = np.loadtxt(path + "exp_Zhao/" + 'YCO.csv', delimiter=';', unpack=True)
xCO2, YCO2ref = np.loadtxt(path + "exp_Zhao/" + 'YCO2.csv', delimiter=';', unpack=True)
xH2, YH2ref = np.loadtxt(path + "exp_Zhao/" + 'YH2.csv', delimiter=';', unpack=True)
xH2O, YH2Oref = np.loadtxt(path + "exp_Zhao/" + 'YH2O.csv', delimiter=';', unpack=True)
xO2, YO2ref = np.loadtxt(path + "exp_Zhao/" + 'YO2.csv', delimiter=';', unpack=True)

# conversion from [m] to [cm]
xC2H4_cm = 100 * xC2H4
xCO_cm = 100 * xCO
xCO2_cm = 100*xCO2
xH2_cm = 100*xH2
xH2O_cm = 100*xH2O
xO2_cm = 100*xO2

# Plot ref measured data from paper
plt.plot(xC2H4_cm, YC2H4ref, 'k+', label='C2H4 ref')
plt.plot(xCO_cm, YCOref, 'kx', label = 'CO ref')
plt.plot(xCO2_cm, YCO2ref, 'kd', label = 'CO2 ref')
plt.plot(xH2_cm,YH2ref, 'k^', label= 'H2 ref')
plt.plot(xH2O_cm,YH2Oref, 'kv', label='H2O ref')
plt.plot(xO2_cm,YO2ref, 'k.', label = 'O2 ref')

plt.text(1.0, 0.008, '$H_2$',fontsize = 10)
plt.text(1.0, 0.049, '$CO_2$',fontsize = 10)
plt.text(0.5, 0.065, '$H_2O$',fontsize = 10)
plt.text(1.0, 0.135, '$CO$',fontsize = 10)
plt.text(0.04, 0.14, '$O_2$',fontsize = 10)
plt.annotate('$C_2H_4$', xy=(0.03, 0.08), xytext=(0.25, 0.092),
             arrowprops=dict(facecolor='black', shrink=0.05, width=0.1, headwidth=0.1))

plt.xlabel('$x [cm]$',fontsize=15)
plt.xticks(np.arange(0.0, 1.5, 0.25))
plt.xlim(0.0,1.5)
plt.ylabel('Mass Fraction',fontsize=15)

plt.ylim((0.0,0.15))
#plt.title('Volume fraction $f_v$ detected as a function of height above burner', fontsize = 8)

#plt.subplots_adjust(top=0.9, bottom=0.1, left=0.15, right=0.95, hspace=0.25, wspace=0.1)

#plt.legend()
plt.subplots_adjust(top=0.9, bottom=0.1, left=0.15, right=0.95, hspace=0.25, wspace=0.1)
plt.savefig('YmainSpecies.jpg', format='jpg', dpi=1000)
plt.show()
