//
//  AgglomerationTensor.hpp
//  npsSolver
//
//  Created by Alexandre Bouaniche on 25/11/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#ifndef AgglomerationTensor_hpp
#define AgglomerationTensor_hpp

#include <stdio.h>

#include <vector>
#include <string>
#include <iostream>
#include <cmath>

#include "Psd.hpp"

class AgglomerationTensor
{
public:
    AgglomerationTensor(Psd* psdPtr);
    
    double getgCoagValue(int iOrigin, int jOrigin, int kDestination);  // with i <= j !!
    double getmAvgFromiOrigin(int iOrigin, int jOrigin, int kDestination);  // with i <= j
    double getmAvgFromjOrigin(int iOrigin, int jOrigin, int kDestination);  // with i <= j 
    
    
protected:
    Psd* m_psdPtr;
    std::vector<std::vector<std::vector<double> > > m_gCoagTensor;
    std::vector<std::vector<std::vector<double> > > m_avgMassiTensor;
    std::vector<std::vector<std::vector<double> > > m_avgMassjTensor;
    // As in thesis D'Aubagnac. vector of "proportion" of mass repartition between 3 "destination" bins. Origin bins: i, j. Destination bins k= (l-1), k= l, k = (l+1) with l the bin containing the middle of [Vmin,i + Vmin,j ; Vmax,i + Vmax,j]
    
    // protected methods
    double gCoagValue(int i, int j, int k);  // i and j colliding bins and k is the destination bin. l is the "centered bin". k non-zero only for k=(l-1); k=l and k=(l+1)
    double miCoagValue(int i, int j, int k);  // avg mass of i which goes to k
    double mjCoagValue(int i, int j, int k);  // avg mass of j which goes to k
    
    double numeratorgCoagLminusOne(int i, int j, int k);
    double numeratorgCoagL(int i, int j, int k);
    double numeratorgCoagLplusOne(int i, int j, int k);
    
    void setgCoagTensor();
    void setavgMassiTensor();
    void setavgMassjTensor();
    
    int findBin(double m);
};

#endif /* AgglomerationTensor_hpp */
