//
//  Grid.cpp
//  soot_stochastic
//
//  Created by Alexandre Bouaniche on 28/08/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#include "Grid.hpp"

#include <iostream>
#include <string>
#include <vector>
#include <cmath>

using namespace std;

// by default a grid of type geometric2 is built, with numberTotCells = 30 and firstCellmConst = 1.0

Grid::Grid()
{
    m_gridType = "geometric2";
    m_numberTotCells = 30;
    m_firstCellmConst = 0.1;
    
    std::vector<GridCell> gridCellVector;
    
    int i = 0;
    for(i=0; i<m_numberTotCells; i++)
    {
        double power = double(i);
        double mConst = m_firstCellmConst * pow(2.0, power);
        double mAvg = 1.125 * mConst;
        double mInf = 0.75 * mConst;
        double mSup = 1.5 * mConst;
        GridCell gridCell = GridCell(mConst, mAvg, mInf, mSup);
        gridCellVector.push_back(gridCell);
    }
    
    m_gridCellVector = gridCellVector;
}


Grid::Grid(string gridType, int numberTotCells, double mSupMax, double factorFirstCell, double mInf, double c2coef)  // for type "standard" (geometricC2 from D'aubagnac thesis") vmaxi = (vmin+vC2) * ( vmax/(vmin+vC2) )^( (i-1)/(nsect-1) )  [for i>1] Here done with mass
{
    if(gridType == "standard")
    {
        m_gridType = "standard";
        m_numberTotCells = numberTotCells;
        std::vector<GridCell> gridCellVector;
        
        //double mInf1 = 1.076E-21; // first cell should be centered around mAvg corresponding to 2 * mDimer = 4 * mPAH = 4 * 16 * cmass = 1.275264E-21 [g]
        
        double mInf1 = mInf;   //
        
        double mC2 = c2coef;
        double mSup1 = mInf1+ factorFirstCell * mC2;  // !!! factorFirstCell  !!!
        double mAvg1 = (mSup1 +mInf1) / 2.0;
        double mConst1 = mAvg1;
        GridCell gridCell1 = GridCell(mConst1, mAvg1, mInf1, mSup1);
        gridCellVector.push_back(gridCell1);
        
        double mInf = mSup1;
        
        int i(2);
        for(i=2; i<(numberTotCells+1);i++)
        {
            double exponant = double(i-1) / double(numberTotCells-1);
            double mSup = (mInf1+factorFirstCell * mC2) * pow(mSupMax / (mInf1+factorFirstCell * mC2),exponant);
            double mAvg = (mSup+mInf)/2;
            double mConst = mAvg;
            GridCell gridCell = GridCell(mConst,mAvg,mInf,mSup);
            gridCellVector.push_back(gridCell);
            mInf = mSup;
        }
        
        m_gridCellVector = gridCellVector;
    }
    else
    {
        cout << "error Grid constructor gridType" << endl;
    }
}

/*
Grid::Grid(string gridType, int numberTotCells, double mSupMax, double factorFirstCell, double mInf)  // for type "standard" (geometricC2 from D'aubagnac thesis") vmaxi = (vmin+vC2) * ( vmax/(vmin+vC2) )^( (i-1)/(nsect-1) )  [for i>1] Here done with mass
{
    if(gridType == "standard")
    {
        m_gridType = "standard";
        m_numberTotCells = numberTotCells;
        std::vector<GridCell> gridCellVector;
        
        //double mInf1 = 1.076E-21; // first cell should be centered around mAvg corresponding to 2 * mDimer = 4 * mPAH = 4 * 16 * cmass = 1.275264E-21 [g]
        
        double mInf1 = mInf;   //
        
        double mC2 = mInf;
        double mSup1 = mInf1+ factorFirstCell * mC2;  // !!! factorFirstCell  !!!
        double mAvg1 = (mSup1 +mInf1) / 2.0;
        double mConst1 = mAvg1;
        GridCell gridCell1 = GridCell(mConst1, mAvg1, mInf1, mSup1);
        gridCellVector.push_back(gridCell1);
        
        double mInf = mSup1;
        
        int i(2);
        for(i=2; i<(numberTotCells+1);i++)
        {
            double exponant = double(i-1) / double(numberTotCells-1);
            double mSup = (mInf1+factorFirstCell * mC2) * pow(mSupMax / (mInf1+factorFirstCell * mC2),exponant);
            double mAvg = (mSup+mInf)/2;
            double mConst = mAvg;
            GridCell gridCell = GridCell(mConst,mAvg,mInf,mSup);
            gridCellVector.push_back(gridCell);
            mInf = mSup;
        }
        
        m_gridCellVector = gridCellVector;
    }
    else
    {
        cout << "error Grid constructor gridType" << endl;
    }
}
*/

/*
Grid::Grid(string gridType, int numberTotCells, double mSupMax, double factorFirstCell)  // for type "standard" (geometricC2 from D'aubagnac thesis") vmaxi = (vmin+vC2) * ( vmax/(vmin+vC2) )^( (i-1)/(nsect-1) )  [for i>1] Here done with mass
{
    if(gridType == "standard")
    {
        m_gridType = "standard";
        m_numberTotCells = numberTotCells;
        std::vector<GridCell> gridCellVector;
        
        //double mInf1 = 1.076E-21; // first cell should be centered around mAvg corresponding to 2 * mDimer = 4 * mPAH = 4 * 16 * cmass = 1.275264E-21 [g]
        
        double mInf1 = 5.98E-22;   // if PAH nucleation: first cell should be centered around mAvg corresponding to 2 * mPAH = 2 * 16 * cmass = 6.37632E-22 [g]
        
        double mC2 = 2*12*1.6605E-24;
        double mSup1 = mInf1+ factorFirstCell * mC2;  // !!! factorFirstCell  !!!
        double mAvg1 = (mSup1 +mInf1) / 2.0;
        double mConst1 = mAvg1;
        GridCell gridCell1 = GridCell(mConst1, mAvg1, mInf1, mSup1);
        gridCellVector.push_back(gridCell1);
        
        double mInf = mSup1;
        
        int i(2);
        for(i=2; i<(numberTotCells+1);i++)
        {
            double exponant = double(i-1) / double(numberTotCells-1);
            double mSup = (mInf1+factorFirstCell * mC2) * pow(mSupMax / (mInf1+factorFirstCell * mC2),exponant);
            double mAvg = (mSup+mInf)/2;
            double mConst = mAvg;
            GridCell gridCell = GridCell(mConst,mAvg,mInf,mSup);
            gridCellVector.push_back(gridCell);
            mInf = mSup;
        }
        
        m_gridCellVector = gridCellVector;
    }
    else
    {
        cout << "error Grid constructor gridType" << endl;
    }
}
*/


// for geometric2

Grid::Grid(std::string gridType, int numberTotCells, double firstCellmAvg) : m_gridType(gridType), m_numberTotCells(numberTotCells)
{
    if(m_gridType == "geometric2")
    {
        m_firstCellmConst = firstCellmAvg / 1.125;
        std::vector<GridCell> gridCellVector;
        
        int i = 0;
        for(i=0; i<m_numberTotCells; i++)
        {
            double power = double(i);
            double mConst = m_firstCellmConst * pow(2.0, power);
            double mAvg = 1.125 * mConst;  // ok only for fs = 2 !!
            double mInf = 0.75 * mConst;   // ok only for fs = 2 !!
            double mSup = 1.5 * mConst;    // ok only for fs = 2 !!
            GridCell gridCell = GridCell(mConst, mAvg, mInf, mSup);
            gridCellVector.push_back(gridCell);
        }
        
        m_gridCellVector = gridCellVector;
    }
    else
    {
        cout << "error Grid constructor gridType" << endl;
        
    }
    
}



//for regular

/*
Grid::Grid(std::string gridType, int numberTotCells, double firstCellmConst, double step) : m_gridType(gridType), m_numberTotCells(numberTotCells), m_firstCellmConst(firstCellmConst)
{
    if(gridType == "regular")
    {
        std::vector<GridCell> gridCellVector;
        
        int i = 0;
        for(i=0; i<m_numberTotCells; i++)
        {
            double mConst = m_firstCellmConst + double(i*step);
            double mAvg = mConst;
            double mInf = mConst - step/2.0;
            double mSup = mConst + step/2.0;
            GridCell gridCell = GridCell(mConst, mAvg, mInf, mSup);
            gridCellVector.push_back(gridCell);
        }
        
        m_gridCellVector = gridCellVector;
    }
    else
    {
        cout << "error Grid constructor gridType" << endl;
    }
}
 */


/*
Grid::Grid(std::string gridType, int numberTotCells, double minInf, double maxSup, double qCoef) : m_gridType(gridType), m_numberTotCells(numberTotCells)
{
    if(gridType == "geometricStandard")
    {
        std::vector<GridCell> gridCellVector;
        
        double mInf = minInf;
        
        // expression from thesis Molenaar p33. expression for L(i+1/2) which corresponds to mSup(i)
        int i=0;
        for(i=0; i<m_numberTotCells; i++)
        {
            int N = m_numberTotCells - 1;
            double power = double (i) - double(N);
            double mSup = minInf + pow(2.0,double(power/qCoef)) * (maxSup - minInf);
            double mAvg = mInf + (mSup - mInf)/2.0;
            double mConst = mAvg;
            
            GridCell gridCell = GridCell(mConst, mAvg, mInf, mSup);
            gridCellVector.push_back(gridCell);
            
            mInf = mSup;
        }
        
        m_gridCellVector = gridCellVector;
    }
    else
    {
        cout << "error Grid constructor gridType" << endl;
    }
}
 */


//input vector with the first mInf, all the mConst and the last mSup

Grid::Grid(std::string gridType, std::vector<double> mConstVector) : m_gridType(gridType)
{
    if(m_gridType == "custom")
    {
        std::vector<GridCell> gridCellVector;
        
        double mInf = mConstVector[0];
        double mSup;
        
        int i=1;
        for(i=1; i<(mConstVector.size()-1); i++)
        {
            double mConst = mConstVector[i];
            
            if(i>1)
            {
                mInf = mConstVector[i-1] + (mConstVector[i] - mConstVector[i-1]) / 2.0;
            }
            else
            {
                mInf = mConstVector[0];
                
            }
            
            if(i<(mConstVector.size()-2))
            {
                mSup = mConstVector[i] + (mConstVector[i+1] - mConstVector[i]) /2.0;
            }
            else
            {
                mSup = mConstVector[i+1];
            }
            
            double mAvg = mInf + (mSup - mInf) / 2.0;
            
            GridCell gridCell = GridCell(mConst, mAvg, mInf, mSup);
            gridCellVector.push_back(gridCell);
            
        }
        
        m_gridCellVector = gridCellVector;
    }
    else
    {
        cout << "error Grid constructor gridType" << endl;
    }
}


Grid::Grid(int numberTotCells, double firstCellmAvg, double fs, string gridType) : m_numberTotCells(numberTotCells), m_gridType(gridType)
{
    // for a geometric grid
    if(gridType == "geometricFs")
    {
        double mInf0 = 2.0* firstCellmAvg / (1.0+fs);
        
        std::vector<GridCell> gridCellVector;
        
        int i = 0;
        for(i=0; i<m_numberTotCells; i++)
        {
            double power = double(i);
            double mInf = mInf0 * pow(fs,power);
            double mSup = mInf0 * pow(fs,power+1);
            double mAvg = (mInf + mSup) / 2.0;
            double mConst = mAvg;
            
            GridCell gridCell = GridCell(mConst, mAvg, mInf, mSup);
            gridCellVector.push_back(gridCell);
        }
        
        m_gridCellVector = gridCellVector;
        
    }
    else if(gridType == "geometricFsGhost")
    {
        double mInf0 = 2.0* firstCellmAvg / (1.0+fs);
        double mInfGhost = 0.9 * mInf0;
        
        std::vector<GridCell> gridCellVector;
        
        int i = 0;
        // first, ghost cell
        double mInf = mInfGhost;
        double mSup = mInf0;
        double mAvg = (mInf + mSup) / 2.0;
        double mConst = mAvg;
        
        GridCell gridCell = GridCell(mConst, mAvg, mInf, mSup);
        gridCellVector.push_back(gridCell);
        //cout << "mAvgGhost = " << mAvg << endl;
        // then, other cells. Total M cells. M-1 following geometricFs + 1 ghost
        for(i=0; i<(m_numberTotCells-1); i++)
        {
            double power = double(i);
            mInf = mInf0 * pow(fs,power);
            mSup = mInf0 * pow(fs,power+1);
            mAvg = (mInf + mSup) / 2.0;
            mConst = mAvg;
            //cout << "mAvg = " << mAvg << endl;
            
            GridCell gridCell = GridCell(mConst, mAvg, mInf, mSup);
            gridCellVector.push_back(gridCell);
        }
        
        m_gridCellVector = gridCellVector;
        
    }
    else
    {
        cout << "ERROR Grid creator, gridType" << endl;
    }
}


Grid::Grid(int numberTotCells, double firstCellmAvg, double fs, string gridType, double gridIncrement) : m_numberTotCells(numberTotCells), m_gridType(gridType)
{
    // for a geometric grid
    /*
     mInf_{i+1} / mInf_{i} = mSup_{i+1} / mSup_{i} = mAvg_{i+1} / mAvg_{i} = deltam_{i+1} / deltam_{i} = fs
     mSup_i = mInf_i+1 = fs mInf_i
     mAvg_i = (1+fs) * mInf_i / 2
     deltam_i = (fs - 1) * mInf_i
     deltam_i = (fs - 1) * 2 / (fs + 1) * mAvg_i
     */
    if(gridType == "geometricFs")
    {
        double mInf0 = 2.0* firstCellmAvg / (1.0+fs);
        
        std::vector<GridCell> gridCellVector;
        
        int i = 0;
        for(i=0; i<m_numberTotCells; i++)
        {
            double power = double(i);
            double mInf = mInf0 * pow(fs,power);
            double mSup = mInf0 * pow(fs,power+1);
            double mAvg = (mInf + mSup) / 2.0;
            double mConst = mAvg;
            
            GridCell gridCell = GridCell(mConst, mAvg, mInf, mSup);
            gridCellVector.push_back(gridCell);
        }
        
        m_gridCellVector = gridCellVector;
        
    }
    else if(gridType == "regular")
    {
        double mInf0 = firstCellmAvg - gridIncrement / 2.0;
        
        std::vector<GridCell> gridCellVector;
        
        int i = 0;
        for(i=0; i<m_numberTotCells; i++)
        {
            double mInf = mInf0 + i * gridIncrement;
            double mSup = mInf0 + (i+1)* gridIncrement;
            double mAvg = (mInf + mSup) / 2.0;
            double mConst = mAvg;
            
            GridCell gridCell = GridCell(mConst, mAvg, mInf, mSup);
            gridCellVector.push_back(gridCell);
        }
        
        m_gridCellVector = gridCellVector;
    }
    else
    {
        cout << "ERROR Grid creator, gridType" << endl;
    }
}


Grid::Grid(int numberTotCells, double firstCellmAvg, double fs, string gridType, double gridIncrement, double alphaGrid, double mMin) : m_numberTotCells(numberTotCells), m_gridType(gridType)
{
    if(gridType == "geometricFs")
    {
        double mInf0 = 2.0* firstCellmAvg / (1.0+fs);
        
        std::vector<GridCell> gridCellVector;
        
        int i = 0;
        for(i=0; i<m_numberTotCells; i++)
        {
            double power = double(i);
            double mInf = mInf0 * pow(fs,power);
            double mSup = mInf0 * pow(fs,power+1);
            double mAvg = (mInf + mSup) / 2.0;
            double mConst = mAvg;
            
            GridCell gridCell = GridCell(mConst, mAvg, mInf, mSup);
            gridCellVector.push_back(gridCell);
        }
        
        m_gridCellVector = gridCellVector;
        
    }
    else if(gridType == "regular")
    {
        double mInf0 = firstCellmAvg - gridIncrement / 2.0;
        
        std::vector<GridCell> gridCellVector;
        
        int i = 0;
        for(i=0; i<m_numberTotCells; i++)
        {
            double mInf = mInf0 + i * gridIncrement;
            double mSup = mInf0 + (i+1)* gridIncrement;
            double mAvg = (mInf + mSup) / 2.0;
            double mConst = mAvg;
            
            GridCell gridCell = GridCell(mConst, mAvg, mInf, mSup);
            gridCellVector.push_back(gridCell);
        }
        
        m_gridCellVector = gridCellVector;
    }
    else if(m_gridType == "exponential")
    {
        std::vector<GridCell> gridCellVector;
        int i(0);
        
        // equation from Rigopoulos 2003 eq(5)
        // in the paper, equation for the first value at i = 1. as we have our first value at i = 0 we will substitue the exponent (i-1) by exponent i.
        for(i=0; i<m_numberTotCells; i++)
        {
            double mInf = mMin + mMin * (1.0 - pow(alphaGrid, double(i)) ) / ( 1.0 - alphaGrid);
            double mSup =mMin + mMin * (1.0 - pow(alphaGrid, double(i+1)) ) / ( 1.0 - alphaGrid);
            double mAvg = ( mInf + mSup ) / 2.0;
            double mConst = mAvg;
            
            GridCell gridCell = GridCell(mConst, mAvg, mInf, mSup);
            gridCellVector.push_back(gridCell);
        }
        
        m_gridCellVector = gridCellVector;
    }
    else
    {
        cout << "ERROR Grid creator, gridType" << endl;
    }
}



void Grid::showGrid()
{
    int i=0;
    
    cout << endl << endl;
    cout << "mConst   mAvg   mInf   mSup" << endl;
    
    for(i=0; i<m_gridCellVector.size();i++)
    {
        GridCell gridCell = m_gridCellVector[i];
        cout << gridCell.getmConst() << "   " << gridCell.getmAvg() << "   " << gridCell.getmInf() << "   " << gridCell.getmSup() << endl;
    }
}


GridCell Grid::getGridCell(int i)
{
    return m_gridCellVector[i];
}


int Grid::getSize()
{
    int size(0);
    int i(0);
    for(i=0; i<m_gridCellVector.size();i++)
    {
        size++;
    }
    return size;
}

