//
//  Grid.hpp
//  soot_stochastic
//
//  Created by Alexandre Bouaniche on 28/08/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#ifndef Grid_hpp
#define Grid_hpp

#include <stdio.h>
#include <vector>
#include <string>

#include "GridCell.hpp"


// Object that represents the grid in the space of sizes (mass). It is constituted of GridCells

class Grid
{
public:
    Grid();  //  geometric2 without specified parameters
    
    Grid(std::string gridType, int numberTotCells, double mSupMax, double factorFirstCell, double mInf, double c2coef); // for type "standard" (geometricC2 from D'aubagnac thesis", with factorFirstCell*mC2 for mSup1)
    
    //Grid(std::string gridType, int numberTotCells, double mSupMax, double factorFirstCell, double mInf); // for type "standard" (geometricC2 from D'aubagnac thesis", with factorFirstCell*mC2 for mSup1). With mInf based on 2 pyrene molecules
    
    //Grid(std::string gridType, int numberTotCells, double mSupMax, double factorFirstCell); // for type "standard"
    
    Grid(std::string gridType, int numberTotCells, double firstCellmAvg); // for type geometric2 with specified parameters
    
    
    //Grid(std::string gridType, int numberTotCells, double minInf, double maxSup, double qCoef);  //for geometricStandard
    
    Grid(std::string gridType, std::vector<double> mConstVector); // For customized
    
    Grid(int numberTotCells, double firstCellmAvg, double fs, std::string gridType); // for type geometricFs (like in Park, Rogak 2004).
    
    Grid(int numberTotCells, double firstCellmAvg, double fs, std::string gridType, double gridIncrement); // for type geometricFs or regular
    
    Grid(int numberTotCells, double firstCellmAvg, double fs, std::string gridType, double gridIncrement, double alphaGrid, double mMin); // for type geometricFs or regular or exponential
    
    
    
    void showGrid();
    //void refineGrid         Idea
    
    GridCell getGridCell(int i);
    
    int getSize();
    
    
protected:
    std::vector<GridCell> m_gridCellVector;
    std::string m_gridType;     // available options: "standard", "regular"; "geometric2"; "geometricStandard"; "custom"; "geometricFs"; "exponential"
    int m_numberTotCells;
    double m_firstCellmConst;
};


#endif /* Grid_hpp */
