//
//  HomogeneousGasPhase.hpp
//  soot_stochastic
//
//  Created by Alexandre Bouaniche on 30/08/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#ifndef HomogeneousGasPhase_hpp
#define HomogeneousGasPhase_hpp

#include <stdio.h>
#include <vector>
#include <string>

#include "streams.hpp"

// array with Y values of relevant species as a function of time. known in advance and homogeneous within the "reactor"

// could be replaced by a cantera object like a reactor or flame

class HomogeneousGasPhase
{
public:
    HomogeneousGasPhase();
    HomogeneousGasPhase(std::string pathProject, std::string pathTarget, std::string dataFilename,std::string labelsFilename);
    
    
    // returns the rank in m_t of the closest (superior) value to argument t
    int getRankt(double t);
    double getValue(double t, std::vector<double> vector);
    
    double getT(double t);
    double getYPAHnucl(double t);
    double getYA1(double t);
    double getYA2(double t);
    double getYA3(double t);
    double getYA4(double t);
    double getYC2H2(double t);
    double getYO2(double t);
    double getYOH(double t);
    
    double getx(double t);
    double getu(double t);
    double getYH2O(double t);
    double getYH2(double t);
    double getYH(double t);
    double getrho(double t);
    double getYA4imposed(double t);
    double getwmA4(double t);
    
    double getValueVectorwmA4(int i);
    double getPperfectGas(double t);  // in Pa
    
    
    double getConcentration(double Yspecies, double rho, double molarMass); // rho in [kg/m3], molarMass in [g/mol]. Output concentration in [part/cm3]
    double getConcentrationString(double Yspecies, double rho, std::string speciesName);
    // in [part/cm3]
    double getMolarConcentration(double t, double rho, std::string speciesName);  // in [mol/cm3]
    
protected:
    
    std::vector<double> m_t;
    std::vector<double> m_T;
    std::vector<double> m_YPAHnucl;
    std::vector<double> m_YA1;
    std::vector<double> m_YA2;
    std::vector<double> m_YA3;
    std::vector<double> m_YA4;
    std::vector<double> m_YC2H2;
    std::vector<double> m_YO2;
    std::vector<double> m_YOH;
    std::vector<double> m_YA4imposed;
    std::vector<double> m_wmA4;
    
    std::vector<double> m_x;
    std::vector<double> m_u;
    std::vector<double> m_YH2O;
    std::vector<double> m_YH2;
    std::vector<double> m_YH;
    std::vector<double> m_rho;
    
    static const double avogadro;
    static const double WA1;
    static const double WA2;
    static const double WA3;
    static const double WA4;
    static const double WC2H2;
    static const double WO2;
    static const double WOH;
    static const double WH2O;
    static const double WH2;
    static const double WH;
    
    // possible development: integrate here a Cantera object reactor/flame...
};

#endif /* HomogeneousGasPhase_hpp */
