//
//  NpSolver.cpp
//  npsSolver
//
//  Created by Alexandre Bouaniche on 08/09/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#include "NpSolver.hpp"

using namespace std;

const double NpSolver::rhoSoot = 1.8; // [g/cm^3]
const double NpSolver::cmass = 12.0 * 1.6605e-24;  // [g]
const double NpSolver::pi = 3.14159265;

NpSolver::NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr) : m_sourcePtr(sourcePtr), m_aeroSolPtr(aeroSolPtr), m_psdPtr(psdPtr), m_itCurrent(0), m_partPerCluster(1), m_writeFreq(1), m_sectionalMethod(""), m_growthParkPtr(nullptr), m_hybridTrigger(0), m_showPsdFreq(1), m_subItWrite(false)
{
    
}

NpSolver::NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr, int itTotal, double timePerIt) : m_sourcePtr(sourcePtr), m_aeroSolPtr(aeroSolPtr), m_psdPtr(psdPtr), m_itTotal(itTotal), m_timePerIt(timePerIt), m_itCurrent(0), m_partPerCluster(1), m_writeFreq(1), m_sectionalMethod(""), m_growthParkPtr(nullptr), m_hybridTrigger(0), m_showPsdFreq(1), m_subItWrite(false)
{
    
}



NpSolver::NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr, int itTotal, double timePerIt, string pathProject, string pathTargetPsd, string pathTargetGlobal) : m_sourcePtr(sourcePtr), m_aeroSolPtr(aeroSolPtr), m_psdPtr(psdPtr), m_itTotal(itTotal), m_timePerIt(timePerIt), m_pathProject(pathProject), m_pathTargetPsd(pathTargetPsd), m_pathTargetGlobal(pathTargetGlobal), m_itCurrent(0), m_partPerCluster(1), m_writeFreq(1), m_sectionalMethod(""), m_growthParkPtr(nullptr), m_hybridTrigger(0), m_showPsdFreq(1), m_subItWrite(false)
{
    
}

NpSolver::NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr, int itTotal, double timePerIt, string pathProject, string pathTargetPsd, string pathTargetGlobal, string solverType) : m_sourcePtr(sourcePtr), m_aeroSolPtr(aeroSolPtr), m_psdPtr(psdPtr), m_itTotal(itTotal), m_timePerIt(timePerIt), m_pathProject(pathProject), m_pathTargetPsd(pathTargetPsd), m_pathTargetGlobal(pathTargetGlobal), m_solverType(solverType), m_itCurrent(0), m_partPerCluster(1), m_writeFreq(1), m_sectionalMethod(""), m_growthParkPtr(nullptr), m_hybridTrigger(0), m_showPsdFreq(1), m_subItWrite(false)
{
    
}

NpSolver::NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr, int itTotal, double timePerIt, string pathProject, string pathTargetPsd, string pathTargetGlobal, string solverType, TimeStep* timeStepPtr, double tFinal, string pbeMethod) : m_sourcePtr(sourcePtr), m_aeroSolPtr(aeroSolPtr), m_psdPtr(psdPtr), m_itTotal(itTotal), m_timePerIt(timePerIt), m_pathProject(pathProject), m_pathTargetPsd(pathTargetPsd), m_pathTargetGlobal(pathTargetGlobal), m_solverType(solverType), m_itCurrent(0), m_timeStepPtr(timeStepPtr), m_timeSum(0.0), m_tFinal(tFinal), m_pbeMethod(pbeMethod), m_tStepGrowth(timePerIt), m_partPerCluster(1), m_writeFreq(1), m_sectionalMethod(""), m_growthParkPtr(nullptr), m_hybridTrigger(0), m_showPsdFreq(1), m_subItWrite(false)
{
    
}


NpSolver::NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr, int itTotal, double timePerIt, string pathProject, string pathTargetPsd, string pathTargetGlobal, string solverType, TimeStep* timeStepPtr, double tFinal, string pbeMethod, int partPerCluster, int writeFreq) : m_sourcePtr(sourcePtr), m_aeroSolPtr(aeroSolPtr), m_psdPtr(psdPtr), m_itTotal(itTotal), m_timePerIt(timePerIt), m_pathProject(pathProject), m_pathTargetPsd(pathTargetPsd), m_pathTargetGlobal(pathTargetGlobal), m_solverType(solverType), m_itCurrent(0), m_timeStepPtr(timeStepPtr), m_timeSum(0.0), m_tFinal(tFinal), m_pbeMethod(pbeMethod), m_tStepGrowth(timePerIt), m_partPerCluster(partPerCluster), m_writeFreq(writeFreq), m_sectionalMethod(""), m_growthParkPtr(nullptr), m_hybridTrigger(0), m_showPsdFreq(1), m_subItWrite(false)
{
    
}

NpSolver::NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr, int itTotal, double timePerIt, string pathProject, string pathTargetPsd, string pathTargetGlobal, string solverType, TimeStep* timeStepPtr, double tFinal, string pbeMethod, int partPerCluster, int writeFreq, string sectionalMethod, GrowthPark* growthParkPtr) : m_sourcePtr(sourcePtr), m_aeroSolPtr(aeroSolPtr), m_psdPtr(psdPtr), m_itTotal(itTotal), m_timePerIt(timePerIt), m_pathProject(pathProject), m_pathTargetPsd(pathTargetPsd), m_pathTargetGlobal(pathTargetGlobal), m_solverType(solverType), m_itCurrent(0), m_timeStepPtr(timeStepPtr), m_timeSum(0.0), m_tFinal(tFinal), m_pbeMethod(pbeMethod), m_tStepGrowth(timePerIt), m_partPerCluster(partPerCluster), m_writeFreq(writeFreq), m_sectionalMethod(sectionalMethod), m_growthParkPtr(growthParkPtr), m_hybridTrigger(0), m_showPsdFreq(1), m_subItWrite(false)
{
    
}

NpSolver::NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr, int itTotal, double timePerIt, string pathProject, string pathTargetPsd, string pathTargetGlobal, string solverType, TimeStep* timeStepPtr, double tFinal, string pbeMethod, int partPerCluster, int writeFreq, string sectionalMethod, GrowthPark* growthParkPtr, int hybridTrigger) : m_sourcePtr(sourcePtr), m_aeroSolPtr(aeroSolPtr), m_psdPtr(psdPtr), m_itTotal(itTotal), m_timePerIt(timePerIt), m_pathProject(pathProject), m_pathTargetPsd(pathTargetPsd), m_pathTargetGlobal(pathTargetGlobal), m_solverType(solverType), m_itCurrent(0), m_timeStepPtr(timeStepPtr), m_timeSum(0.0), m_tFinal(tFinal), m_pbeMethod(pbeMethod), m_tStepGrowth(timePerIt), m_partPerCluster(partPerCluster), m_writeFreq(writeFreq), m_sectionalMethod(sectionalMethod), m_growthParkPtr(growthParkPtr), m_hybridTrigger(hybridTrigger), m_showPsdFreq(1), m_subItWrite(false)
{
    
}

NpSolver::NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr, int itTotal, double timePerIt, string pathProject, string pathTargetPsd, string pathTargetGlobal, string solverType, TimeStep* timeStepPtr, double tFinal, string pbeMethod, int partPerCluster, int writeFreq, string sectionalMethod, GrowthPark* growthParkPtr, int hybridTrigger, int showPsdFreq) : m_sourcePtr(sourcePtr), m_aeroSolPtr(aeroSolPtr), m_psdPtr(psdPtr), m_itTotal(itTotal), m_timePerIt(timePerIt), m_pathProject(pathProject), m_pathTargetPsd(pathTargetPsd), m_pathTargetGlobal(pathTargetGlobal), m_solverType(solverType), m_itCurrent(0), m_timeStepPtr(timeStepPtr), m_timeSum(0.0), m_tFinal(tFinal), m_pbeMethod(pbeMethod), m_tStepGrowth(timePerIt), m_partPerCluster(partPerCluster), m_writeFreq(writeFreq), m_sectionalMethod(sectionalMethod), m_growthParkPtr(growthParkPtr), m_hybridTrigger(hybridTrigger), m_showPsdFreq(showPsdFreq), m_subItWrite(false)
{
    
}

NpSolver::NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr, int itTotal, double timePerIt, string pathProject, string pathTargetPsd, string pathTargetGlobal, string solverType, TimeStep* timeStepPtr, double tFinal, string pbeMethod, int partPerCluster, int writeFreq, string sectionalMethod, GrowthPark* growthParkPtr, int hybridTrigger, int showPsdFreq, bool subItWrite, double subItWrite_tmin, double subItWrite_tmax) : m_sourcePtr(sourcePtr), m_aeroSolPtr(aeroSolPtr), m_psdPtr(psdPtr), m_itTotal(itTotal), m_timePerIt(timePerIt), m_pathProject(pathProject), m_pathTargetPsd(pathTargetPsd), m_pathTargetGlobal(pathTargetGlobal), m_solverType(solverType), m_itCurrent(0), m_timeStepPtr(timeStepPtr), m_timeSum(0.0), m_tFinal(tFinal), m_pbeMethod(pbeMethod), m_tStepGrowth(timePerIt), m_partPerCluster(partPerCluster), m_writeFreq(writeFreq), m_sectionalMethod(sectionalMethod), m_growthParkPtr(growthParkPtr), m_hybridTrigger(hybridTrigger), m_showPsdFreq(showPsdFreq), m_subItWrite(subItWrite), m_subItWrite_tmin(subItWrite_tmin), m_subItWrite_tmax(subItWrite_tmax)
{
    
}

void NpSolver::consolePrint()
{
    if(m_solverType == "analytical")
    {
        consolePrintAnalytical();
    }
    else if (m_solverType == "HomogeneousGasPhase")
    {
        consolePrintHomogeneousGasPhase();
    }
    else
    {
        cout << "ERROR: solverType" << endl;
    }
}

void NpSolver::consolePrintHomogeneousGasPhase()
{
    //printing Initial information
    double t=getTime();
    
    
    cout << "iteration " << m_itCurrent << "   t = " << t << endl;
    cout << "nvT = " << m_psdPtr->getNvT() << "   npTot = " << m_psdPtr->getNpT() << endl;
    cout << "dotAt = " << partAggloSourceTot(t) << "   dotH = " << m_sourcePtr->partNuclSource(t) << endl;
    cout << "dotAtPerIt = " << partAggloSourceTot(t)*m_timePerIt << "   dotHperIt = " << m_sourcePtr->partNuclSource(t) * m_timePerIt << endl;
    cout << "YA4' = " << m_sourcePtr->YA4corrected(t) << "   consoTot = "<< m_sourcePtr->getConsoTotPAH() << endl << endl;
     
}

void NpSolver::consolePrintAnalytical()
{
    double t=getTime();
    
    cout << "iteration " << m_itCurrent << "   t = " << t << endl;
    cout << "nvT = " << m_psdPtr->getNvT() << "   npTot = " << m_psdPtr->getNpT() << endl;
    cout << "dotAt = " << partAggloSourceTot(t) << "   dotH = " << m_sourcePtr->partNuclSource(t) << endl;
    cout << "dotAtPerIt = " << partAggloSourceTot(t)*m_timePerIt << "   dotHperIt = " << m_sourcePtr->partNuclSource(t) * m_timePerIt << endl << endl;
}

void NpSolver::solve()
{
    cout << "NP Soot Solver" << endl << endl;
    
    writeIntegralFile();
    consolePrint();
    
    m_psdPtr->showPsd();
    
    int i(0);
    
    if(m_timeStepPtr->tStepType() == "constant")
    {
        for(i=0;i<m_itTotal;i++)
        {
            advanceIteration();
        }
    }
    else if( (m_timeStepPtr->tStepType() == "fixedAlpha") || (m_timeStepPtr->tStepType() == "growth") || (m_timeStepPtr->tStepType() == "alphaAndgSubsteps") )
    {
        while(m_timeSum < m_tFinal)
        {
            advanceIteration();
        }
    }
    else
    {
        cout << "ERROR= tStepType" << endl;
    }
    
    consolePrint();
    m_psdPtr->showPsd();
    writePsd();
    writeIntegralLine();
    
}

void NpSolver::advanceIteration()
{
    if(m_pbeMethod == "npBasic" || m_pbeMethod == "npResidual" || m_pbeMethod == "npBasicPart" || m_pbeMethod == "npResidualPart" || m_pbeMethod == "hybrid" || m_pbeMethod == "hybridPart")
    {
        advanceIterationStochastic();
    }
    else if(m_pbeMethod == "sectional")
    {
        advanceIterationSectional();
    }
}


void NpSolver::advanceIterationStochastic()
{
    double t = getTime();
    
    // calculating and setting the time step
    calcAndSetTimeStep(t);
    
    //printing information at some iterations
    int it = m_itCurrent;
    consolePrint();
    
    if(it%m_writeFreq == 0)
    {
        writePsd();
        writeIntegralLine();
    }
    
    if(it%m_showPsdFreq == 0)
    {
        m_psdPtr->showPsd();
    }
    
    
    // advancing one global iteration (one nucl+agglo iteration and possibly several growth subiterations)
    
    if(m_timeStepPtr->tStepType() == "alphaAndgSubsteps")
    {
        //double tsPrevious = m_timePerIt;
        //double ratio = m_timeStepPtr->ratioTstepAggloGrowth(t, tsPrevious);
        double ratio = m_timeStepPtr->ratioTstepAggloGrowth(t);
        int i(0);
        
        double subTime(0.0);
        
        for(i=0;i<ratio;i++)
        {
            advanceGrowth();  // based on m_tStepGrowth

            m_psdPtr->countAndSetAllNp();
            m_psdPtr->calcAndSetAllNv();
            m_psdPtr->calcAndSetAllmAvgVar();
            check_reequilibrate_all_bins();
            m_psdPtr->calcAndSetAllmAvgVar();
            
            
            subTime+=m_tStepGrowth;
            
            // writing additional writePsd, for growth subIteration within one agglo iteration. Just for a specified time range
            if(m_subItWrite == true)
            {
                double t2 = t + subTime;
                if(t2 >= m_subItWrite_tmin && t2 <= m_subItWrite_tmax)
                {
                    //writePsd(double t2) in order to write t2 in the name of the .dat file
                    writePsdSubIt(t2);
                }
            }
        }
        //cout << "subItTime = " << subTime << endl;
    }
    else
    {
        advanceGrowth();
        
        m_psdPtr->countAndSetAllNp();
        m_psdPtr->calcAndSetAllNv();
        
        m_psdPtr->calcAndSetAllmAvgVar();
        
        check_reequilibrate_all_bins();
        
        m_psdPtr->calcAndSetAllmAvgVar();
    }
    
    
    advancePBE();
    // takes value of nvT from PSD
    // reallocate
    // calculate and set nvT(t+dt) in PSD
    
    if(m_solverType == "HomogeneousGasPhase")
    {
        //Summing over time mass [g/cm3] of pyrene consumed by dimerization. used to correct YA4 to YA4'
        correctionYA4();
    }
    
    //m_psdPtr->countAndSetAllNp(); now done in reallocate which is run in advancePBE (need to do it at this moment for npResidual)
    m_psdPtr->calcAndSetAllNv();  // calcAndSetAllNv still located here as nvT needs to be updated before and it is done in advancePBE after reallocate.
    
    iterate();
}

void NpSolver::advanceIterationSectional()
{
    double t = getTime();
    
    // calculating and setting the time step
    calcAndSetTimeStep(t);  // setting tSteps in NpSolver
    
    //printing information at some iterations
    int it = m_itCurrent;
    consolePrint();
    
    if(it%m_writeFreq == 0)
    {
        writePsd();
        writeIntegralLine();
    }
    
    
    // advancing one global iteration (one nucl+agglo iteration and possibly several growth subiterations)
    
    if(m_timeStepPtr->tStepType() == "alphaAndgSubsteps")
    {
        //double tsPrevious = m_timePerIt;
        //double ratio = m_timeStepPtr->ratioTstepAggloGrowth(t, tsPrevious);
        double ratio = m_timeStepPtr->ratioTstepAggloGrowth(t);
        int i(0);
        
        double subTime(0.0);
        
        for(i=0;i<ratio;i++)
        {
            advanceGrowth();  // based on m_tStepGrowth
            //m_psdPtr->countAndSetAllNp();
            //m_psdPtr->calcAndSetAllNv();
            subTime+=m_tStepGrowth;
            
            // writing additional writePsd, for growth subIteration within one agglo iteration. Just for a specified time range
            if(m_subItWrite == true)
            {
                double t2 = t + subTime;
                if(t2 >= m_subItWrite_tmin && t2 <= m_subItWrite_tmax)
                {
                    //writePsd(double t2) in order to write t2 in the name of the .dat file
                    writePsdSubIt(t2);
                }
            }
        }
        //cout << "subItTime = " << subTime << endl;
    }
    else
    {
        advanceGrowth();
        //m_psdPtr->countAndSetAllNp();
        //m_psdPtr->calcAndSetAllNv();
    }
    
    
    advancePBE();
    // takes value of nvT from PSD
    // reallocateSectional
    // calculate and set nvT(t+dt) in PSD
    
    if(m_solverType == "HomogeneousGasPhase")
    {
        //Summing over time mass [g/cm3] of pyrene consumed by dimerization. used to correct YA4 to YA4'
        correctionYA4();
    }
    
    //m_psdPtr->countAndSetAllNp(); now done in reallocate which is run in advancePBE (need to do it at this moment for npResidual)
    //m_psdPtr->calcAndSetAllNv();  // calcAndSetAllNv still located here as nvT needs to be updated before and it is done in advancePBE after reallocate.
    
    iterate();
    
}

void NpSolver::calcAndSetTimeStep(double t)
{
    if(m_timeStepPtr->tStepType() == "fixedAlpha")
    {
        //double tsPrevious = m_timePerIt;
        //double timeStep = m_timeStepPtr->aggloTimeStep(t, tsPrevious);
        double timeStep = m_timeStepPtr->aggloTimeStep(t);
        
        m_timePerIt = timeStep;
        m_tStepGrowth = m_timePerIt;
        
    }
    else if(m_timeStepPtr->tStepType() == "constant")
    {
        m_tStepGrowth = m_timePerIt;
        double timeStep = m_tStepGrowth;
        cout << "tsConstant = " << timeStep << endl <<endl;
    }
    else if(m_timeStepPtr->tStepType() == "growth")
    {
        double timeStep = m_timeStepPtr->growthTargetTimeStep(t);
        cout << "tsGrowth = " << timeStep << endl << endl;
        m_timePerIt = timeStep;
        m_tStepGrowth = m_timePerIt;
        
    }
    else if(m_timeStepPtr->tStepType() == "alphaAndgSubsteps")
    {
        //double tsPrevious = m_timePerIt;
        //double agglotimeStep = m_timeStepPtr->aggloTimeStep(t, tsPrevious);
        double agglotimeStep = m_timeStepPtr->aggloTimeStep(t);
        
        m_timePerIt = agglotimeStep;
        
        //double growthTimeStep = m_timeStepPtr->finalGrowthTimeStep(t, tsPrevious);
        double growthTimeStep = m_timeStepPtr->finalGrowthTimeStep(t);
        m_tStepGrowth = growthTimeStep;
        
        //double ratio = m_timeStepPtr->ratioTstepAggloGrowth(t, tsPrevious);
        double ratio = m_timeStepPtr->ratioTstepAggloGrowth(t);
        cout << "tsAgglo = " << agglotimeStep << "   tsGrowth = " << growthTimeStep << "   ratio = " << ratio << endl << endl;
    }
    else
    {
        cout << "ERROR tStepType" << endl;
    }
}


void NpSolver::advanceGrowth()
{
    if(m_pbeMethod == "npBasic")
    {
        advanceGrowthStochasticBin();
    }
    else if (m_pbeMethod == "npBasicPart")
    {
        advanceGrowthStochasticPart();
    }
    else if(m_pbeMethod == "sectional")
    {
        advanceGrowthSectional();
    }
    else if(m_pbeMethod == "npResidual")
    {
        advanceGrowthStochasticBin();
        advanceGrowthSectional();
    }
    else if(m_pbeMethod == "npResidualPart")
    {
        advanceGrowthStochasticPart();
        advanceGrowthSectional();
    }
    else if(m_pbeMethod == "hybrid")
    {
        advanceGrowthSectionalHybrid();
        
        advanceGrowthStochasticBinHybrid();
    }
    else if(m_pbeMethod == "hybridPart")
    {
        advanceGrowthSectionalHybrid();
        advanceGrowthStochasticPartHybrid();
    }
    else
    {
        cout << "ERROR pbeMethod advanceGrowth" << endl;
    }
    
}

void NpSolver::advanceGrowthStochasticBin()
{
    double t = getTime();
    int i(0);
    int npTot = m_aeroSolPtr->getNpTot();
    vector<double> deltamVec = m_sourcePtr->deltamTotGrowthVector(t);
    
    double sumDeltamAllPart(0.0); // to have global information on growth
    //double deltamAvgPerPart(0.0); // avg per stochastic particles
    
    for(i=0;i<npTot;i++)
    {
        double m = m_aeroSolPtr->getMass(i);
        int bini = findBin(m);
        
        // source term per second
        double deltam = deltamVec[bini];
        
        // global for all stochastic particles
        sumDeltamAllPart+= deltam;
        
        // source term per it
        //double deltamPerIt = deltam * m_timePerIt;
        double deltamPerIt = deltam * m_tStepGrowth;
        
        //checking that mass particle is not beeing set at a lower value than minimum value of the grid before adding/substracting mass
        double mInfGrid = m_psdPtr->getmInf(0);
        
        if(m+deltamPerIt>mInfGrid)
        {
            m_aeroSolPtr->addMass(i, deltamPerIt);
        }
        else
        {
            cout << "ERROR: m < mInfGrid" <<endl;
        }
    }
}


void NpSolver::advanceGrowthStochasticBinHybrid()
{
    double t = getTime();
    int i(0);
    int npTot = m_aeroSolPtr->getNpTot();
    vector<double> deltamVec = m_sourcePtr->deltamTotGrowthVector(t);
    
    double sumDeltamAllPart(0.0); // to have global information on growth
    //double deltamAvgPerPart(0.0); // avg per stochastic particles
    
    // find the bins for which npl is under the trigger -> growthStochastic will not be executed in these bins
    vector<int> underTriggerBins;
    
    int nBins = m_psdPtr->getSize();
    int bin(0);
    for(bin=0;bin<nBins; bin++)
    {
        int npl = m_psdPtr->getNp(bin);
        if(npl < m_hybridTrigger)
        {
            underTriggerBins.push_back(bin);
        }
        
    }
    
    
    for(i=0;i<npTot;i++)
    {
        double m = m_aeroSolPtr->getMass(i);
        int bini = findBin(m);
        
        // test if bini is a bin which is in underTriggerBins
        if(isIntInlist(bini, underTriggerBins))
        {
            // the particle is in a bin which is in underTriggerBins vector which means growth will not be handled through stochastic particles -> do nothing
        }
        else
        {
            // source term per second
            double deltam = deltamVec[bini];
            
            // global for all stochastic particles
            sumDeltamAllPart+= deltam;
            
            // source term per it
            //double deltamPerIt = deltam * m_timePerIt;
            double deltamPerIt = deltam * m_tStepGrowth;
            
            //checking that mass particle is not beeing set at a lower value than minimum value of the grid before adding/substracting mass
            double mInfGrid = m_psdPtr->getmInf(0);
            
            if(m+deltamPerIt>mInfGrid)
            {
                m_aeroSolPtr->addMass(i, deltamPerIt);
            }
            else
            {
                /*
                cout << "deltam = " << deltam << endl;
                cout << "ERROR NpSolver::advanceGrowthStochasticBinHybrid : m < mInfGrid" <<endl;
                cout << "error2" << endl;
                 */
            }
        }
        
    }
}


void NpSolver::advanceGrowthStochasticPartHybrid()
{
    double t = getTime();
    int i(0);
    int npTot = m_aeroSolPtr->getNpTot();
    
    // find the bins for which npl is under the trigger -> growthStochastic will not be executed in these bins
    vector<int> underTriggerBins;
    
    int nBins = m_psdPtr->getSize();
    int bin(0);
    for(bin=0;bin<nBins; bin++)
    {
        int npl = m_psdPtr->getNp(bin);
        if(npl < m_hybridTrigger)
        {
            underTriggerBins.push_back(bin);
        }
        
    }
    
    
    for(i=0;i<npTot;i++)
    {
        double m = m_aeroSolPtr->getMass(i);
        int bini = findBin(m);
        
        // test if bini is a bin which is in underTriggerBins
        if(isIntInlist(bini, underTriggerBins))
        {
            // the particle is in a bin which is in underTriggerBins vector which means growth will not be handled through stochastic particles -> do nothing
        }
        else
        {
            // source term per second
            double deltam = m_sourcePtr->deltamTotGrowthPart(t, m);
            
            // source term per it
            //double deltamPerIt = deltam * m_timePerIt;
            double deltamPerIt = deltam * m_tStepGrowth;
            
            //checking that mass particle is not beeing set at a lower value than minimum value of the grid before adding/substracting mass
            double mInfGrid = m_psdPtr->getmInf(0);
            
            if(m+deltamPerIt>mInfGrid)
            {
                m_aeroSolPtr->addMass(i, deltamPerIt);
            }
            else
            {
                cout << "ERROR: m < mInfGrid" <<endl;
                cout << "i = " << i << "   m = " << m << "   deltamPerIt = " << deltamPerIt << "   deltam = " << deltam << endl;
            }
        }
        
    }
}



void NpSolver::advanceGrowthStochasticPart()
{
    double t = getTime();
    int i(0);
    int npTot = m_aeroSolPtr->getNpTot();
    
    double sumDeltamAllPart(0.0); // to have global information on growth
    //double deltamAvgPerPart(0.0); // avg per stochastic particles
    
    for(i=0;i<npTot;i++)
    {
        double m = m_aeroSolPtr->getMass(i);
        
        // source term per second
        double deltam = m_sourcePtr->deltamTotGrowthPart(t, m);
        
        // global for all stochastic particles
        sumDeltamAllPart+= deltam;
        
        // source term per it
        //double deltamPerIt = deltam * m_timePerIt;
        double deltamPerIt = deltam * m_tStepGrowth;
        
        //checking that mass particle is not beeing set at a lower value than minimum value of the grid before adding/substracting mass
        double mInfGrid = m_psdPtr->getmInf(0);
        
        if(m+deltamPerIt>mInfGrid)
        {
            m_aeroSolPtr->addMass(i, deltamPerIt);
        }
        else
        {
            cout << "ERROR: m < mInfGrid" <<endl;
            cout << "NpSolver::advanceGrowthStochasticPart" << endl;
            cout << "m = " << m << "   deltam = " << deltam << endl;
        }
    }
}


void NpSolver::advanceGrowthSectional()
{
    int nBins = m_psdPtr->getSize();
    int i(0);
    
    // dotNvResidual values need to be calculated first for all bins and then NvResidual is modified in all bins. If calculated and modified for one bin, then for the following bin there is an error as Nvi(i-1) was modified
    vector<double> dotNvResidualVector;
    
    //double dotNvGrowthAllBins(0.0);
    //double dotmTotGrowthAllBins(0.0);
    
    for(i=0; i<nBins; i++)
    {
        double dotNvResiduali(0.0);
        
        if(m_sectionalMethod == "park")
        {
            double t = getTime();
            dotNvResiduali = m_growthParkPtr->dotNvi(i, t);  // [part.cm-3.s-1]
            dotNvResidualVector.push_back(dotNvResiduali);
            //cout << "NpSolver::advanceGrowthSectional: dotNvResidual = " << dotNvResiduali << endl;
        }
        else if(m_sectionalMethod == "2pointUpwindmAvg" || m_sectionalMethod == "2pointUpwindmSup")
        {
            dotNvResiduali = dotNv2pointUpwind(i);  // [part.cm-3.s-1]
            dotNvResidualVector.push_back(dotNvResiduali);
        }
        else
        {
            cout << "ERROR NpSolver::advanceGrowthSectional. sectionalMethod error" << endl;
        }
        

        
    }
    
    // <<endl << "dotmtotAllBins = " << dotmTotGrowthAllBins << endl;
    //cout <<endl << "dotNvGrowthAllBins = " << dotNvGrowthAllBins << endl << endl;
    
    for(i=0; i<nBins; i++)
    {
        double deltaNvResidualPerIt = dotNvResidualVector[i] * m_tStepGrowth;  // [part.cm-3]
        
        Bin* binPtr = m_psdPtr->getBinPtr(i);
        
        binPtr->addNvResidual(deltaNvResidualPerIt);
    }
}



void NpSolver::advanceGrowthSectionalHybrid()
{
    vector<double> additionalResidual = additionalResidualPerBin();
    addResidualUnderTrigger(additionalResidual);
    
    advanceGrowthSectional();
    
    
    substractResidualUnderTrigger(additionalResidual);
}



void NpSolver::advancePBE()
{
    // tMinusOne
    double t = getTime();
    double nvT = m_psdPtr->getNvT();
    
    // source terms per second
    double dotH = m_sourcePtr->partNuclSource(t);
    double dotAt = partAggloSourceTot(t);
    
    // source terms per iteration
    
    double dotHperIt = dotH * m_timePerIt;
    double dotAtPerIt = dotAt * m_timePerIt;
    
    double dotHplusAtPerIt = dotHperIt + dotAtPerIt;
    
    
    vector<double> alphaVecPerIt = alphaVectorPerIt(dotHplusAtPerIt, dotHperIt);  // calculated at t (tMinusOne) calculating within the alphaVector function nvT(t+dt) used at the denominator of alphaCoefs
    
    
    // proceeding with nucleation and coagulation (alphaVector) reallocations
    if(m_pbeMethod == "npBasic" || m_pbeMethod == "npResidual" || m_pbeMethod == "npBasicPart" || m_pbeMethod == "npResidualPart" || m_pbeMethod == "hybrid" || m_pbeMethod == "hybridPart")
    {
        reallocateStochastic(alphaVecPerIt, dotHplusAtPerIt);
        check_reequilibrate_all_bins();
        m_psdPtr->calcAndSetAllmAvgVar();
    }
    else if(m_pbeMethod == "sectional")
    {
        reallocateSectional(alphaVecPerIt, dotHplusAtPerIt);
    }
    else
    {
        cout << "ERROR pbeMethod in advancePBE" << endl;
    }
    
    
    
    
    // t (t+dt)     advancing nvT on Psd object.
    double nvTplusDt(0.0);
    nvTplusDt = nvT + dotHplusAtPerIt;
    
    m_psdPtr->setNvt(nvTplusDt);     // nvT(t+dt)
    
    
}





double NpSolver::getTime()
{
    double time(0.0);
    
    if(m_timeStepPtr->tStepType() == "constant")
    {
        time = m_itCurrent * m_timePerIt;
    }
    else if( (m_timeStepPtr->tStepType() == "fixedAlpha") || (m_timeStepPtr->tStepType() == "growth") || (m_timeStepPtr->tStepType() == "alphaAndgSubsteps") )
    {
        time = m_timeSum;
    }
    else
    {
        cout << "ERROR: tStepType" << endl;
    }
    
    return time;
}

int NpSolver::getIteration()
{
    return m_itCurrent;
}

void NpSolver::setItTotal(int itTotal)
{
    m_itTotal = itTotal;
}


void NpSolver::iterate()
{
    m_itCurrent+= 1;
    m_timeSum += m_timePerIt;
}



vector<double> NpSolver::alphaVectorPerIt(double dotHplusAtPerIt, double dotHperIt)
{
    double t = getTime();  // tMinusOne
    
    // all agglo source terms calculated for tMinusOne (t in the formalism description). Per second
    vector<double> partAggloSourceVec = m_sourcePtr->partAggloSourceVector(t);
    
    
    // all agglo source terms per It
    vector<double> partAggloSourceVecPerIt;
    int i(0);
    for(i=0;i<partAggloSourceVec.size();i++)
    {
        double dotAlPerIt = partAggloSourceVec[i] * m_timePerIt;
        partAggloSourceVecPerIt.push_back(dotAlPerIt);
    }
    
    // nvTPlusDt = nT(t+dt) in formalism description is used at the denominator of alphaVector
    double nvT = m_psdPtr->getNvT();
    double nvTPlusDt = nvT + dotHplusAtPerIt;
    
    vector<double> alphaVec;
    double alpha0(1.0);
    
    alpha0 = (dotHperIt + partAggloSourceVecPerIt[0]) / nvTPlusDt;
    
    alphaVec.push_back(alpha0);
    
    double alpha(1.0);
    
    for(i=1;i<partAggloSourceVec.size();i++)
    {
        alpha = partAggloSourceVecPerIt[i] / nvTPlusDt;
        alphaVec.push_back(alpha);
    }
    
    return alphaVec;
}


double NpSolver::partAggloSourceTot(double t)
{
    double dotAt(0.0);
    vector<double> partAggloSource = m_sourcePtr->partAggloSourceVector(t);
    
    int i(0);
    for(i=0;i<partAggloSource.size();i++)
    {
        dotAt+=partAggloSource[i];
    }
    return dotAt;
}


void NpSolver::reallocateStochastic(vector<double> alphaVecPerIt, double dotHplusAtPerIt)
{
    double nvT = m_psdPtr->getNvT();
    double nvTplusDt = nvT + dotHplusAtPerIt;
    double alphaHplusAtPerIt = dotHplusAtPerIt / nvTplusDt;
    
    int npTot = m_psdPtr->getNpT();
    double nplDtCalc(0.0);
    int nplDtRounded(0);
    int npltMinus(0);
    
    int deltaNpl(0);
    vector<int> deltaNplVec;
    vector<double> nplDtCalcVec;
    
    // Calculating deltaNpl
    int i(0);
    for(i=0;i<alphaVecPerIt.size();i++)
    {
        npltMinus = m_psdPtr->getNp(i);   // PSD must "count" np(t) after growth before reallocate
        
        if(m_pbeMethod == "npBasic" || m_pbeMethod == "npBasicPart")
        {
            nplDtCalc = double(npltMinus) * (1.0 - alphaHplusAtPerIt) + alphaVecPerIt[i] * double(npTot);
        }
        else if (m_pbeMethod == "npResidual" || m_pbeMethod == "npResidualPart" || m_pbeMethod == "hybrid" || m_pbeMethod == "hybridPart")
        {
            Bin* binPtr = m_psdPtr->getBinPtr(i);
            double nvResidualMinus = binPtr->getNvResidual();
            double alphaResidual = nvResidualMinus / nvTplusDt;
            
            nplDtCalc = double(npltMinus) * (1.0 - alphaHplusAtPerIt) + (alphaVecPerIt[i] + alphaResidual) * double(npTot);
            nplDtCalcVec.push_back(nplDtCalc);
            
            //cout << "section: " << i << "   nplDtCalc = " << nplDtCalc << "   npltMinus = " << npltMinus << "   1.0 - alphaHplusAtPerIt = " << (1.0 - alphaHplusAtPerIt) << "   (alphaVecPerIt[i] + alphaResidual) = " << (alphaVecPerIt[i] + alphaResidual) << endl;
        }
        else
        {
            cout << "ERROR pbeMethod" << endl;
            nplDtCalc = 0.0;
        }
        
        
        nplDtRounded = rounding(nplDtCalc);
        
        // additional check for consuming more particles than present in the bin
        if(nplDtRounded <0)
        {
            cout << "more particles consumed than present un Bin " << i << endl;
            nplDtRounded = 0;
        }
        
        deltaNpl = nplDtRounded - npltMinus;
        
        deltaNplVec.push_back(deltaNpl);
        
    }
    
    //cout << "deltaNpl[0] = " << deltaNplVec[0] << endl;
    
    vector<vector<int> > decreasingBinsVec;
    vector<vector<int> > increasingBinsVec;
    
    int partPosTot(0);
    int partNegTot(0);
    
    
    // separating information on decreasing Bins and increasing Bins
    
    for(i=0;i<deltaNplVec.size();i++)
    {
        if(deltaNplVec[i]<0)
        {
            vector<int> binRankAndDeltaNpl;
            binRankAndDeltaNpl.push_back(i);
            binRankAndDeltaNpl.push_back(deltaNplVec[i]);
            
            decreasingBinsVec.push_back(binRankAndDeltaNpl);
            partNegTot-= deltaNplVec[i];
            
        }
        else if (deltaNplVec[i] >0)
        {
            vector<int> binRankAndDeltaNpl;
            binRankAndDeltaNpl.push_back(i);
            binRankAndDeltaNpl.push_back(deltaNplVec[i]);
            
            increasingBinsVec.push_back(binRankAndDeltaNpl);
            partPosTot+=deltaNplVec[i];
        }
    }
    
    cout << endl << endl;
    cout << "number of stochastic particles to be reset = " << partNegTot << endl;
    cout << "number of values to be set on stochastic particles = " << partPosTot << endl << endl;
    
    
    vector<GasParticle*> partToResetPtrVec;
    
    
    // getting a list of pointers to the particles of size binRank and picking particles using random list without duplicate  (see comments further)
    
    for(i=0;i<decreasingBinsVec.size();i++)
    {
        int binRank = decreasingBinsVec[i][0];           // binRank: rank of bin previously stored in decreasingBinsVec from deltaNplVec
        
        double mInf = m_psdPtr->getmInf(binRank);    // defining inferior and superior limits of bin of rank binRank
        
        double mSup = m_psdPtr->getmSup(binRank);
        
        vector<GasParticle*> partOfBinRankPtrVec;
        
        int npTot = m_aeroSolPtr->getNpTot();
        int j(0);
        for(j=0;j<npTot;j++)
        {
            GasParticle* gasPartPtr = m_aeroSolPtr->getGasPartPtr(j);
            double mass = gasPartPtr->getMass();
            if(mass >= mInf && mass < mSup)
            {
                partOfBinRankPtrVec.push_back(gasPartPtr);         // collecting pointers on all particles of size within bin of rank binRank
            }
        }
        
        int absDelta = -decreasingBinsVec[i][1];  //absDelta = -deltaNpl(binRank) > 0
        int npl = m_psdPtr->getNp(binRank);
        int min1 = absDelta;               // usually absDelta should be <= npl but if absDelta > npl  (test below) then, warning message and min1 = npl. Then nbPicked = min1 so that we don't try to pick more particles than are available in one bin.
        
        if(npl!=partOfBinRankPtrVec.size())       // check if npl info in PSD is the same as count of particles vector of size within the bin done above.
        {
            cout << "ERROR np for Bin of mAvg = " << m_psdPtr->getmAvg(binRank) << endl << endl;
        }
        
        if(absDelta > npl)
        {
            cout << "more particles consumed than present in Bin of mAvg = " << m_psdPtr->getmAvg(binRank) << endl << endl;
            min1 = npl;
        }
        
        
        // picking particles using random list without duplicate
        
        if(npl>1)
        {
            double t = getTime();
            int nbPicked = min1;   // minimum between absDelta and npl
            int maxVal = npl - 1;   // maximum rank that can be picked
            vector<int> ranksPicked = randomListWithoutDuplicate(t, nbPicked, maxVal);
            
            j=0;
            for(j=0; j<ranksPicked.size();j++)
            {
                int rankInPartOfBinVec = ranksPicked[j];
                GasParticle* pickedPart = partOfBinRankPtrVec[rankInPartOfBinVec];
                
                partToResetPtrVec.push_back(pickedPart);   // picked particle (pointer) added in partToResetPtrVec
            }
        }
        else if (npl == 1)
        {
            GasParticle* pickedPart = partOfBinRankPtrVec[0];
            partToResetPtrVec.push_back(pickedPart);
        }
        else //npl < 1 . There can be no particule to reset in this bin
        {
            cout << "ERROR in NpSolver::reallocateStochastic. Trying to reset a particle from an empty bin" << endl;
        }
    }
    
    
    
    //checking minimum between partNegTot and partPosTot
    
    int min2 = partNegTot;
    if(partPosTot == partNegTot)
    {
        cout << "OK same number of values to set and particles to be reset" << endl << endl;
    }
    else if (partNegTot > partPosTot)
    {
        cout << "partNegTot > partPosTot" << endl << endl;
        min2 = partPosTot;
    }
    else if (partNegTot < partPosTot)
    {
        cout << "partNegTot < partPosTot" << endl << endl;
    }
    
    
    
    // now resetting !!
    int countResetDone(0);
    
    for(i=0;i<increasingBinsVec.size();i++)
    {
        int rankBinToSet = increasingBinsVec[i][0];
        int nbToSet = increasingBinsVec[i][1];
        
        // !! WIP !! NOT used in CES
        double mInf = m_psdPtr->getmInf(rankBinToSet);
        double mSup = m_psdPtr->getmSup(rankBinToSet);
        double deltam = mSup - mInf;
        
        // account for uncentered mAvgVariDestination!!
        // M1 correction
        // !! WIP !! used in CES
        double mAvgVarPrevious = m_psdPtr->getmAvgVar(rankBinToSet);
        
        // for piecewise_linear pdf reallocation. deltanInf and deltanSup needed
        double deltanp = double(deltaNplVec[rankBinToSet]);  // delta np in the section (at representative size)
        double deltan = deltanp / npTot * nvTplusDt / deltam; // corresponding delta n
        
        
        // interpolate nInf and nSup at the borders of the section. Specific cases for first and last bin
        double deltanInf(0.0);
        double deltanSup(0.0);
        if(rankBinToSet == 0)
        {
            // for first bin: use only the slope with iPlusOne. Mass conservation of nucleation? mAvgVar(0) should stay at mAvgConst?
            // get variables of neighbouring cells
            double deltanpPlusOne = double(deltaNplVec[rankBinToSet + 1]);
            //double deltanpMinusOne = double(deltaNplVec[rankBinToSet - 1]);
            
            double mSupPlusOne = m_psdPtr->getmSup(rankBinToSet + 1);
            double mInfPlusOne = mSup;
            //double mSupMinusOne = mInf;
            //double mInfMinusOne = m_psdPtr->getmInf(rankBinToSet - 1);
            
            double deltamPlusOne = mSupPlusOne - mInfPlusOne;
            //double deltamMinusOne = mSupMinusOne - mInfMinusOne;
            
            double deltanPlusOne = deltanpPlusOne / npTot * nvTplusDt / deltamPlusOne;
            //double deltanMinusOne = deltanpMinusOne / npTot * nvTplusDt / deltamMinusOne;
            
            double mAvgVarPreviousPlusOne = m_psdPtr->getmAvgVar(rankBinToSet + 1);
            //double mAvgVarPreviousMinusOne = m_psdPtr->getmAvgVar(rankBinToSet - 1);
            
            // interpolate n
            deltanInf = deltan + (deltanPlusOne - deltan) / (mAvgVarPreviousPlusOne - mAvgVarPrevious) * (mInf - mAvgVarPrevious);
            deltanSup = deltan + (deltanPlusOne - deltan) / (mAvgVarPreviousPlusOne - mAvgVarPrevious) * (mSup - mAvgVarPrevious);
            
            
        } else if(rankBinToSet == (m_psdPtr->getSize()-1))
        {
            // get variables of neighbouring cells
            //double deltanpPlusOne = double(deltaNplVec[rankBinToSet + 1]);
            double deltanpMinusOne = double(deltaNplVec[rankBinToSet - 1]);
            
            //double mSupPlusOne = m_psdPtr->getmSup(rankBinToSet + 1);
            //double mInfPlusOne = mSup;
            double mSupMinusOne = mInf;
            double mInfMinusOne = m_psdPtr->getmInf(rankBinToSet - 1);
            
            //double deltamPlusOne = mSupPlusOne - mInfPlusOne;
            double deltamMinusOne = mSupMinusOne - mInfMinusOne;
            
            //double deltanPlusOne = deltanpPlusOne / npTot * nvTplusDt / deltamPlusOne;
            double deltanMinusOne = deltanpMinusOne / npTot * nvTplusDt / deltamMinusOne;
            
            //double mAvgVarPreviousPlusOne = m_psdPtr->getmAvgVar(rankBinToSet + 1);
            double mAvgVarPreviousMinusOne = m_psdPtr->getmAvgVar(rankBinToSet - 1);
            
            // interpolate n
            deltanInf = deltanMinusOne + (deltan - deltanMinusOne) / (mAvgVarPrevious - mAvgVarPreviousMinusOne) * (mInf - mAvgVarPreviousMinusOne);
            deltanSup = deltan + (deltan - deltanMinusOne) / (mAvgVarPrevious - mAvgVarPreviousMinusOne) * (mSup - mAvgVarPrevious);
            
        } else
        {
            // get variables of neighbouring cells
            double deltanpPlusOne = double(deltaNplVec[rankBinToSet + 1]);
            double deltanpMinusOne = double(deltaNplVec[rankBinToSet - 1]);
            
            double mSupPlusOne = m_psdPtr->getmSup(rankBinToSet + 1);
            double mInfPlusOne = mSup;
            double mSupMinusOne = mInf;
            double mInfMinusOne = m_psdPtr->getmInf(rankBinToSet - 1);
            
            double deltamPlusOne = mSupPlusOne - mInfPlusOne;
            double deltamMinusOne = mSupMinusOne - mInfMinusOne;
            
            double deltanPlusOne = deltanpPlusOne / npTot * nvTplusDt / deltamPlusOne;
            double deltanMinusOne = deltanpMinusOne / npTot * nvTplusDt / deltamMinusOne;
            
            double mAvgVarPreviousPlusOne = m_psdPtr->getmAvgVar(rankBinToSet + 1);
            double mAvgVarPreviousMinusOne = m_psdPtr->getmAvgVar(rankBinToSet - 1);
            
            // interpolate n
            // separate cases where deltanPlusOne and/or deltanMinusOne is 0
            if (deltanPlusOne <= 0.0 && deltanMinusOne > 0.0) {
                deltanInf = deltanMinusOne + (deltan - deltanMinusOne) / (mAvgVarPrevious - mAvgVarPreviousMinusOne) * (mInf - mAvgVarPreviousMinusOne);
                
                
                // mSup is moved to mAvgVar if neighbouring cell has no positive deltan
                deltanSup = deltan;
                mSup = mAvgVarPrevious;
                
                // faire mSup = mAvgVarPrevious ou proche de mAvgVar sup et ne pas allouer au dela. (calculer croisement avec 0?)
            } else if (deltanMinusOne <= 0.0 && deltanPlusOne > 0.0) {
                
                mInf = mAvgVarPrevious;
                deltanInf = deltan;
                
                deltanSup = deltan + (deltanPlusOne - deltan) / (mAvgVarPreviousPlusOne - mAvgVarPrevious) * (mSup - mAvgVarPrevious);
            } else if (deltanMinusOne <= 0.0 && deltanPlusOne <= 0.0) {
                deltanInf = deltan;
                deltanSup = deltan;
            } else { // ie both deltanPlusOne and deltanMinusOne are > 0
                deltanInf = deltanMinusOne + (deltan - deltanMinusOne) / (mAvgVarPrevious - mAvgVarPreviousMinusOne) * (mInf - mAvgVarPreviousMinusOne);
                deltanSup = deltan + (deltanPlusOne - deltan) / (mAvgVarPreviousPlusOne - mAvgVarPrevious) * (mSup - mAvgVarPrevious);
            }
            
            
        }
        
        
        int j(0);
        for(j=0;j<nbToSet;j++)
        {
            if(countResetDone < min2)  // check if the max number of reset was reached (if all the particles to reset were already reset)
            {
                double newMass;
                
                // for the first bin, nucleate at left of the bin, depending on the time step and G
                if (mInf == m_psdPtr->getmInf(0))
                {
                    // uniform
                    //newMass = frand_a_b(mInf, mSup);
                    
                    // subgrid distribution in the first section in cste / v^theta ?
                    double t = getTime();
                    double gmInf = m_sourcePtr->deltamTotGrowthPart(t, mInf);
                    double deltamInf = gmInf * m_timePerIt;
                    newMass = frand_a_b(mInf, (mInf + deltamInf));
                    
                } else
                {
                    //newMass = frand_a_b(mInf, mSup);
                    //newMass = mAvgVarPrevious;
                    
                    // ftac: piecewise linear distribution based on deltan, not n.
                    newMass = piecewise_linear_distrib_n(mInf, mSup, deltanInf, deltanSup);
                    
                }
                
                
                GasParticle* partPtr = partToResetPtrVec[countResetDone];
                //cout << "old mass = " << partPtr->getMass() << "   newMass = " << newMass << endl;
                partPtr->setMass(newMass);
                countResetDone++;
            }
        }
    }
    
    cout << "number of reset done = " << countResetDone << endl << endl;
    
    
    // countAndSetAllNp  done at this moment for npResidual. Before was done outside reallocate function but now it is done here for this purpose and not repeated after.
    m_psdPtr->countAndSetAllNp();
    
    // check for nvResidual after reallocation.
    // check and correct mass difference with mAvgVarPrevious. Should not be changed by reallocation, only by growth.
    if(m_pbeMethod == "npResidual" || m_pbeMethod == "npResidualPart" || m_pbeMethod == "hybrid" || m_pbeMethod == "hybridPart")
    {
        
        i = 0;
        int nBins = m_psdPtr->getSize();
        // check and correct average mass per section. mAvgVarPrevious should not be changed by reallocation, only by growth.
        vector<double> massCorrection;
        vector<double> residualMassCorrection(nBins, 0.0);
        //double sumMassCorrection(0.0);
        double sumResidualMassCorrection(0.0);
        
        for(i = 0; i < nBins; i++)
        { // fill massCorrection vector
            double mAvgVarPrevious = m_psdPtr->getmAvgVar(i);
            double mAvgVarStoch = m_psdPtr->getmAvgVarStoch(i);
            
            double deltamRatio = mAvgVarPrevious / mAvgVarStoch;
            //int npi = m_psdPtr->getNp(i);
            //double deltamSection = (mAvgVarPrevious - mAvgVarStoch) * double(npi) / double(npTot) * nvTplusDt;
            
            massCorrection.push_back(deltamRatio);
            //massCorrection.push_back(deltamSection);
            //sumMassCorrection += deltamSection;
        }
        
        // Apply the correction to the particles (if does not make it go out of the section)
        int part_idx(0);
        for(part_idx = 0; part_idx < npTot; part_idx++)
        {
            double m = m_aeroSolPtr->getMass(part_idx);
            int bini = findBin(m);
            double deltamRatio = massCorrection[bini]; // deltamCorrection should be a proportion instead of a constant in each section.
            
            // WIP nucl + growth
            if(bini == 0)
            {
                deltamRatio = 0.0;
            }
            
            double mCorrected = deltamRatio * m;
            
            //checking that mass particle is not beeing set at a lower value than minimum value of the grid before adding/substracting mass
            //double mInfGrid = m_psdPtr->getmInf(0);
            
            double mInf = m_psdPtr->getmInf(bini);
            double mSup = m_psdPtr->getmSup(bini);
            
            
            // add mass correction only if the particle stays in the section?
            
            /*
             if(bini == 19){
             cout << "part mass = " << m << "   deltam = " << deltam << "   mInf = " << mInf << "   mSup = " << mSup << endl;
             }*/
            
            
            if( (mCorrected) >= mInf & (mCorrected) <= mSup ) {
                //m_aeroSolPtr->addMass(part_idx, deltam);
                m_aeroSolPtr->setMass(part_idx, mCorrected);
                
                /*
                 if(bini == 19){
                 cout << "part mass after correction = " << m + deltam << endl;
                 }*/
            } else {
                // compensate for mass that was not corrected because was going out of bounds through a transfer of NvR between neighboring sections?
                double residualMass = residualMassCorrection[bini];
                double deltaResidualMass = 1.0 / double(npTot) * nvTplusDt * (mCorrected - m); // nb of real part per stoch part * correction  of the stoch part that could not be applied
                residualMass += deltaResidualMass;
                residualMassCorrection[bini] = residualMass;  // update the value of the residualMassCorrection vector
                
                // for printing
                sumResidualMassCorrection += deltaResidualMass;
            }
        }
        
        //cout << "sumResidualMassCorrection = " << sumResidualMassCorrection << endl;
        
        
        // check and set NvResidual (first value, without accounting for residualMass
        
        //cout << "target sumMassCorrection = " << sumMassCorrection << endl;
        
        double sumDeltaNvR(0.0); // only for printing
        double sumDeltamBins(0.0); // only for printing
        for(i=0;i<nBins;i++)
        {
            Bin* binPtr = m_psdPtr->getBinPtr(i);
            int npl = binPtr->getNp();
            double nplExact = nplDtCalcVec[i];
            double deltaNpResidual = nplExact - npl;
            double nvResidual = deltaNpResidual / npTot * nvTplusDt;
            //binPtr->setNvResidual(nvResidual);
            
            //***************************************************************
            // second value of NvResidual to account for residualMass
            
            //double deltaNvR = deltaNvR_from_residualMassCorrection(i, massCorrection);
            double deltaNvR = deltaNvR_from_residualMassCorrection(i, residualMassCorrection);
            nvResidual += deltaNvR;
            binPtr->setNvResidual(nvResidual);
            
            // for printing
            double mAvgVar = m_psdPtr->getmAvgVar(i);
            double deltamBin = deltaNvR * mAvgVar;
            //cout << " i = " << i << "   deltaNvR = " << deltaNvR << "   deltamBin = " << deltamBin << endl;
            sumDeltaNvR += deltaNvR;  // only for printing
            sumDeltamBins += deltamBin;
            //**************************************************************
        }
        //cout << endl << "sumDeltaNvR = " << sumDeltaNvR << "   sumDeltamBins = " << sumDeltamBins << endl;
        
        
        i = 0;
        for (i=0;i<nBins;i++) { // update mAvgVar in another loop as all NvR must be updated before
            // accounting for mAvgVarR different from mAvgVarStoch
            Bin* binPtr = m_psdPtr->getBinPtr(i);
            int npl = binPtr->getNp();
            
            double mAvgVarGlobalPrevious = m_psdPtr->getmAvgVar(i);
            double mAvgVarRupdated = mAvgVarGlobalPrevious;
            binPtr->setmAvgVarR(mAvgVarRupdated);
            
            // update mAvgVarGlobal after updating mAvgVarR
            //double NvStoch = double(npl) / double(npTot) * nvT;
            double NvStoch = double(npl) / double(npTot) * nvTplusDt;
            
            double mAvgVarStoch = m_psdPtr->getmAvgVarStoch(i);
            double nvResidual = binPtr->getNvResidual();
            
            double mNucleation = m_psdPtr->getmAvg(0);
            double mAvgVarGlobalUpdated = m_psdPtr->calcmAvgVar(NvStoch, mAvgVarStoch, nvResidual, mAvgVarRupdated, i, mNucleation);
            binPtr->setmAvgVar(mAvgVarGlobalUpdated);
            //cout << "section: " << i << "   mAvgVarPrevious = " << mAvgVarGlobalPrevious << "   mAvgVarRupdated = " << mAvgVarRupdated << "   mAvgVarStoch = " << mAvgVarStoch << "   mAvgVarUpdated = " << mAvgVarGlobalUpdated << endl;
            //cout << "NvStoch = " << NvStoch << "   NvR = " << nvResidual << endl;
        }
        
        // update NvResidual accounting for residual mass
        
    }
    
}


void NpSolver::reallocateSectional(vector<double> alphaVecPerIt, double dotHplusAtPerIt)
{
    if(m_pbeMethod == "sectional")
    {
        double nvT = m_psdPtr->getNvT();
        double nvTplusDt = nvT + dotHplusAtPerIt;
        
        // calculating deltaNvl
        
        int i(0);
        for(i=0;i<alphaVecPerIt.size();i++)
        {
            Bin* binPtr = m_psdPtr->getBinPtr(i);
            binPtr->setNp(0);
            
            double NvMinus = binPtr->getNv();
            double NvResidual(0.0);
            
            //nplDtCalc = double(npltMinus) * (1.0 - alphaHplusAtPerIt) + alphaVecPerIt[i] * double(npTot);
            
            // different equation
            
            // Nv(t+dt) = Nv(t) + alphaVecPerIt[i] * NvT(t+dt);
            
            NvResidual = NvMinus + alphaVecPerIt[i] * nvTplusDt;
            
            binPtr->setNvResidual(NvResidual);
            
        }
    }
    
    else
    {
        cout << "ERROR pbeMethod" << endl;
    }
    
    
}


int NpSolver::findBin(double m)
{
    int i(0);
    int bin(0);
    int nBins = m_psdPtr->getSize();
    int iMax = nBins-1;
    double mSupMax = m_psdPtr->getmSup(iMax);
    if(m<=mSupMax)
    {
        double mSup = m_psdPtr->getmSup(i);
        while(m>mSup)
        {
            i++;
            mSup = m_psdPtr->getmSup(i);
        }
        bin = i;
    }
    else
    {
        bin = 0;
        cout << "ERROR findBin, mass looked for > mMax" << endl;
    }
    return bin;
}


void NpSolver::writePsd()
{
    if(m_solverType == "analytical")
    {
        writePsdAnalytical();
    }
    else if(m_solverType == "HomogeneousGasPhase")
    {
        writePsdHomogeneousGasPhase();
    }
    else
    {
        cout << "ERROR: solverType" << endl;
    }
    
    writePsdClusters();
}

void NpSolver::writePsdSubIt(double t2)
{
    if(m_solverType == "analytical")
    {
        writePsdAnalyticalSubIt(t2);
    }
    else if(m_solverType == "HomogeneousGasPhase")
    {
        writePsdHomogeneousGasPhaseSubIt(t2);
    }
    else
    {
        cout << "ERROR: solverType" << endl;
    }
    
    writePsdClusters();
}



void NpSolver::writePsdClusters()
{
    if(m_partPerCluster>1)
    {
        if(m_solverType == "analytical")
        {
            writePsdClustersAnalytical(m_partPerCluster);
        }
        else if(m_solverType == "HomogeneousGasPhase")
        {
            writePsdClustersHomogeneousGasPhase(m_partPerCluster);
            //writePsdClustersAnalytical(m_partPerCluster);
        }
        else
        {
            cout << "ERROR: solverType in function writePsdClusters" << endl;
        }
    }
    else
    {}
    
    
}


void NpSolver::writePsdClustersSubIt(double t2)
{
    if(m_partPerCluster>1)
    {
        if(m_solverType == "analytical")
        {
            writePsdClustersAnalyticalSubIt( m_partPerCluster,  t2);
        }
        else if(m_solverType == "HomogeneousGasPhase")
        {
            writePsdClustersHomogeneousGasPhaseSubIt( m_partPerCluster,  t2);
            //writePsdClustersAnalyticalSubIt( m_partPerCluster,  t2);
        }
        else
        {
            cout << "ERROR: solverType in function writePsdClusters" << endl;
        }
    }
    else
    {}
    
    
}



void NpSolver::writeIntegralFile()
{
    if(m_solverType == "HomogeneousGasPhase")
    {
        writeIntegralFileHomogeneousGasPhase();
    }
    else if (m_solverType == "analytical")
    {
        
    }
    else
    {
        cout << "ERROR: solverType" << endl;
    }
}

void NpSolver::writeIntegralFileHomogeneousGasPhase()
{
    string pathProject = m_pathProject;
    string pathTarget = m_pathTargetGlobal;
    string finalPath = pathProject.append(pathTarget);
    string dat = ".dat";
    finalPath = finalPath.append(dat);
    ofstream stream1(finalPath.c_str());
    if(stream1) // error test
    {
        stream1 <<"#global integral or average values along time. Colum labels: 1:it []   2:t [s]   3:x [m]   4:NvT [part/cm3]   5:NvTdetected [part/cm3]   6:dAvgPsd [nm]   7:dMedian [nm]   8:Ys []   9:fv []   10:dotAt [part/cm3/s]   11:dotH [part/cm3/s]   12:YA4 []   13:YA4' []   14:consoTot [g/cm3]   15:alphaHaca []   16:dmedianDetected [nm]   17:fvDetected []   18:YA4hybrid []   19:xshifted [m]   20:dotPAHNucl [g/cm3/s]   21:dotCondTot [molec/cm3/s]   22:rho [g/cm3]   23:CmA4   24:dotA4gas" <<endl;
        
    }
    else
    {
        cout << "ERROR: Impossible to open the file." << endl;
    }
    
}

void NpSolver::writeIntegralLine()
{
    if(m_solverType == "HomogeneousGasPhase")
    {
        writeIntegralLineHomogeneousGasPhase();
    }
    else if (m_solverType == "analytical")
    {
        
    }
    else
    {
        cout << "ERROR: solverType" << endl;
    }
}


void NpSolver::writePsdHomogeneousGasPhase()
{
    // #iteration ..  t = ..  x = ..  NvT = ... column labels:
    
    string pathProject = m_pathProject;
    string pathTarget = m_pathTargetPsd;
    
    HomogeneousGasPhase* gasPhasePtr = m_sourcePtr->getGasPhasePtr();
    double t = getTime();
    int it = m_itCurrent;
    double NvTot = m_psdPtr->getNvT();
    double npTot = m_psdPtr->getNpT();
    
    double x(0.0);
    double xshifted(0.0);
    
    x = gasPhasePtr->getx(t);
    xshifted = x + 0.0025;
    
    string finalPath = pathProject.append(pathTarget);
    
    
    double time = t;
    stringstream ss;
    ss << time;
    string strt = ss.str();
    
    
    finalPath = finalPath.append(strt);
    string dat = ".dat";
    finalPath = finalPath.append(dat);
    
    ofstream stream1(finalPath.c_str());
    if(stream1) // error test
    {
        stream1 << "#PSD File. It = " << it << "   t = " << t << "   x = " << x << "   xshifted = " << xshifted << "   NvT = " << NvTot << "   NpT = " << npTot << "   column labels: 1:mAvg [g]   2:dAvg [nm]   3:np [stochastic part]   4:Nv [part/cm3]   5:nd = dN/dlogd = Nv / (logdSup - logdInf) [part/cm3/size]   6:ndExpNorm = nd(d>3) / Nvtot(d>3) [size-1]   7:alphaHacaBini   8:dcjNano   9:kfCoef   10:betaSelfAggloSpherej   11:betaSelfAggloDcj   12:betaSelfAggloHarmonic   13:betaCondSpherej   14:betaCondDcj   15:betaCondharmonic   16:surfSphere   17:surfFractal   18:kn   19:ndmj   20:ndmjStoch   21:NvR   22:deltamj   23:mInfj   24:mSupj   25:mAvgVar   26:mAvgVarStoch   27:mAvgVarR   28:t"<<endl;
        int j;
        double NvTotDetected = m_psdPtr->calculateNvTotDetected();
        int nbBins = m_psdPtr->getSize();
        
        for(j=0; j<nbBins; j++)
        {
            double mAvgj = m_psdPtr->getmAvg(j);
            double dAvgj = m_psdPtr->getdAvgNano(j);
            double npj = m_psdPtr->getNp(j);
            double Nvj = m_psdPtr->getNv(j);
            double ndj = m_psdPtr->calculateNd(j);
            double ndmj = m_psdPtr->calculateNm(j);  // dNv / dm  ( or dN / dlog(m)  )
            double ndmjStoch = m_psdPtr->calculateNmStochastic(j);
            double ndExpNorm;
            double alphaHacaBini = m_sourcePtr->alphaHacaBin(j, t);
            

            
            if(dAvgj>=m_psdPtr->getSmallestDetectedDiameter())
            {
                ndExpNorm = ndj / NvTotDetected;
            }
            else
            {
                ndExpNorm = 0.0;
            }
            
            double dcj = m_sourcePtr->collisionDiameterCalc(mAvgj);  // in cm
            double dcjNano = dcj * 1.0e7;
            
            double T(0.0);
            double P(0.0);

            T = gasPhasePtr->getT(t);
            P = gasPhasePtr->getPperfectGas(t);
            
            double betaSelfAggloSpherej = m_sourcePtr->betaFree(mAvgj, mAvgj, T);
            double betaSelfAggloDcj = m_sourcePtr->betaFreeDc(mAvgj, mAvgj, dcj, dcj, T);
            double betaSelfAggloHarmonic = m_sourcePtr->betaHarmonicDc(mAvgj, mAvgj, dcj, dcj, T, P);
            
            double mPAH = 16.0*cmass;
            double vPAH = mPAH / rhoSoot;  // in [cm3]
            double oneThird = 1.0/3.0;
            double d3 = vPAH*6.0/pi;
            double dcPAH = pow(d3, oneThird); // in [cm] from sphere for dimers/pyrene
            
            double betaCondSpherej = m_sourcePtr->betaCond(mPAH, mAvgj, T);
            double betaCondDcj = m_sourcePtr->betaCondDc(mPAH, mAvgj, dcj, T);
            double betaCondharmonic = m_sourcePtr->betaHarmonicDc(mPAH, mAvgj, dcPAH, dcj, T, P);
            
            double kf = m_sourcePtr->kfInterpolation(j);
            double df = m_sourcePtr->getDf();
            double kfCoef = 1.0/pow(kf, 1.0/df);
            
            double surfFractal = m_sourcePtr->surfFromFractal(j);
            double surfSphere = m_psdPtr->getsAvg(j);
            
            
            double kn = m_sourcePtr->knudsen(T, P, dcj);
            
            Bin* binPtr = m_psdPtr->getBinPtr(j);
            double NvR = binPtr->getNvResidual();
            
            double mInfj = m_psdPtr->getmInf(j);
            double mSupj = m_psdPtr->getmSup(j);
            double deltamj = mSupj - mInfj;
            
            double mAvgVar = m_psdPtr->getmAvgVar(j);
            double mAvgVarStoch = m_psdPtr->getmAvgVarStoch(j);
            double mAvgVarR = m_psdPtr->getmAvgVarR(j);
            
            
            
            stream1 << mAvgj << "   " << dAvgj << "   " << npj << "   " << Nvj << "   " << ndj << "   "  << ndExpNorm << "   " << alphaHacaBini << "   " << dcjNano << "   " << kfCoef << "   " << betaSelfAggloSpherej << "   " << betaSelfAggloDcj << "   " << betaSelfAggloHarmonic << "   " << betaCondSpherej << "   " << betaCondDcj << "   " << betaCondharmonic << "   " << surfSphere << "   " << surfFractal << "   " << kn << "   " << ndmj << "   " << ndmjStoch << "   " << NvR << "   " << deltamj << "   " << mInfj << "   " << mSupj << "   " << mAvgVar << "   " << mAvgVarStoch << "   " << mAvgVarR  << "   " << t << endl;
        }
        stream1 << endl;
    }
    else
    {
        cout << "ERROR: Impossible to open the file." << endl;
    }
    
}

void NpSolver::writePsdHomogeneousGasPhaseSubIt(double t2)
{
    // #iteration ..  t = ..  x = ..  NvT = ... column labels:
    // 1:mAvg [g]   2:dAvg [nm]   3:np [stochastic part]   4:Nv [part/cm3]   5:nd = dN/dlogd = Nv / (logdSup - logdInf)   6:nd(d>3) / Nvtot(d>3)
    
    string pathProject = m_pathProject;
    string pathTarget = m_pathTargetPsd;
    
    HomogeneousGasPhase* gasPhasePtr = m_sourcePtr->getGasPhasePtr();
    //double t = getTime();
    int it = m_itCurrent;
    double NvTot = m_psdPtr->getNvT();
    double npTot = m_psdPtr->getNpT();
    
    double x(0.0);
    double xshifted(0.0);
    
    //x = gasPhasePtr->getx(t);
    x = gasPhasePtr->getx(t2);
    xshifted = x + 0.0025;
    
    string finalPath = pathProject.append(pathTarget);
    
    
    //double time = t;
    stringstream ss;
    //ss << time;
    ss << t2;
    string strt = ss.str();
    
    
    finalPath = finalPath.append(strt);
    string dat = "_subIt.dat";
    finalPath = finalPath.append(dat);
    
    ofstream stream1(finalPath.c_str());
    if(stream1) // error test
    {
        stream1 << "#PSD File. It = " << it << "   t = " << t2 << "   x = " << x << "   xshifted = " << xshifted << "   NvT = " << NvTot << "   NpT = " << npTot << "   column labels: 1:mAvg [g]   2:dAvg [nm]   3:np [stochastic part]   4:Nv [part/cm3]   5:nd = dN/dlogd = Nv / (logdSup - logdInf) [part/cm3/size]   6:ndExpNorm = nd(d>3) / Nvtot(d>3) [size-1]   7:alphaHacaBini   8:dcjNano   9:kfCoef   10:betaSelfAggloSpherej   11:betaSelfAggloDcj   12:betaSelfAggloHarmonic   13:betaCondSpherej   14:betaCondDcj   15:betaCondharmonic   16:surfSphere   17:surfFractal   18:kn   19:ndmj   20:ndmjStoch   21:NvR   22:deltamj   23:mInfj   24:mSupj   25:mAvgVar   26:mAvgVarStoch   27:mAvgVarR   28:t2"  <<endl;
        int j;
        double NvTotDetected = m_psdPtr->calculateNvTotDetected();
        int nbBins = m_psdPtr->getSize();
        
        for(j=0; j<nbBins; j++)
        {
            double mAvgj = m_psdPtr->getmAvg(j);
            double dAvgj = m_psdPtr->getdAvgNano(j);
            double npj = m_psdPtr->getNp(j);
            double Nvj = m_psdPtr->getNv(j);
            double ndj = m_psdPtr->calculateNd(j);
            double ndmj = m_psdPtr->calculateNm(j);  // dNv / dm  ( or dN / dlog(m)  )
            double ndmjStoch = m_psdPtr->calculateNmStochastic(j);
            double ndExpNorm;
            double alphaHacaBini = m_sourcePtr->alphaHacaBin(j, t2);
            
            if(dAvgj>=m_psdPtr->getSmallestDetectedDiameter())
            {
                ndExpNorm = ndj / NvTotDetected;
            }
            else
            {
                ndExpNorm = 0.0;
            }
            
            double dcj = m_sourcePtr->collisionDiameterCalc(mAvgj);  // in cm
            double dcjNano = dcj * 1.0e7;
            
            double T(0.0);
            double P(0.0);
            
            T = gasPhasePtr->getT(t2);
            P = gasPhasePtr->getPperfectGas(t2);
            
            double betaSelfAggloSpherej = m_sourcePtr->betaFree(mAvgj, mAvgj, T);
            double betaSelfAggloDcj = m_sourcePtr->betaFreeDc(mAvgj, mAvgj, dcj, dcj, T);
            double betaSelfAggloHarmonic = m_sourcePtr->betaHarmonicDc(mAvgj, mAvgj, dcj, dcj, T, P);
            
            double mPAH = 16.0*cmass;
            double vPAH = mPAH / rhoSoot;  // in [cm3]
            double oneThird = 1.0/3.0;
            double d3 = vPAH*6.0/pi;
            double dcPAH = pow(d3, oneThird); // in [cm] from sphere for dimers/pyrene
            
            double betaCondSpherej = m_sourcePtr->betaCond(mPAH, mAvgj, T);
            double betaCondDcj = m_sourcePtr->betaCondDc(mPAH, mAvgj, dcj, T);
            double betaCondharmonic = m_sourcePtr->betaHarmonicDc(mPAH, mAvgj, dcPAH, dcj, T, P);
            
            double kf = m_sourcePtr->kfInterpolation(j);
            double df = m_sourcePtr->getDf();
            double kfCoef = 1.0/pow(kf, 1.0/df);
            
            double surfFractal = m_sourcePtr->surfFromFractal(j);
            double surfSphere = m_psdPtr->getsAvg(j);
            
            
            double kn = m_sourcePtr->knudsen(T, P, dcj);
            
            
            Bin* binPtr = m_psdPtr->getBinPtr(j);
            double NvR = binPtr->getNvResidual();
            
            double mInfj = m_psdPtr->getmInf(j);
            double mSupj = m_psdPtr->getmSup(j);
            double deltamj = mSupj - mInfj;
            
            double mAvgVar = m_psdPtr->getmAvgVar(j);
            double mAvgVarStoch = m_psdPtr->getmAvgVarStoch(j);
            double mAvgVarR = m_psdPtr->getmAvgVarR(j);
            
            
            stream1 << mAvgj << "   " << dAvgj << "   " << npj << "   " << Nvj << "   " << ndj << "   "  << ndExpNorm << "   " << alphaHacaBini << "   " << dcjNano << "   " << kfCoef << "   " << betaSelfAggloSpherej << "   " << betaSelfAggloDcj << "   " << betaSelfAggloHarmonic << "   " << betaCondSpherej << "   " << betaCondDcj << "   " << betaCondharmonic << "   " << surfSphere << "   " << surfFractal << "   " << kn << "   " << ndmj << "   " << ndmjStoch << "   " << NvR << "   " << deltamj << "   " << mInfj << "   " << mSupj << "   " << mAvgVar << "   " << mAvgVarStoch << "   " << mAvgVarR  << "   " << t2 << endl;
        }
        stream1 << endl;
    }
    else
    {
        cout << "ERROR: Impossible to open the file." << endl;
    }
    
}

void NpSolver::writePsdAnalytical()
{
    // #iteration ..  t = ..  x = ..  NvT = ... column labels:
    // 1:mAvg [g]   2:dAvg [nm]   3:np [stochastic part]   4:Nv [part/cm3]   5:nd = dN/dlogd = Nv / (logdSup - logdInf)   6:nd(d>3) / Nvtot(d>3)
    
    string pathProject = m_pathProject;
    string pathTarget = m_pathTargetPsd;
    
    double t = getTime();
    int it = m_itCurrent;
    double NvTot = m_psdPtr->getNvT();
    double npTot = m_psdPtr->getNpT();
    
    string finalPath = pathProject.append(pathTarget);
    
    
    double time = t;
    stringstream ss;
    ss << time;
    string strt = ss.str();
    
    
    finalPath = finalPath.append(strt);
    string dat = ".dat";
    finalPath = finalPath.append(dat);
    
    ofstream stream1(finalPath.c_str());
    if(stream1) // error test
    {
        stream1 << "#PSD File. It = " << it << "   t = " << t  << "   NvT = " << NvTot << "   NpT = " << npTot << "   column labels: 1:mAvg [g]   2:np [stochastic part]   3:Nv [part/cm3]   4:ndm = dN/dm = Nv / (mSup - mInf) [part/cm3/size]   5:ndmStoch (ndm only by stochastic particles, without residual   6:NvR [part/cm3] (residual only)   7:deltamCell [g]   8:mInfCell [g]   9:mSupCell [g]   10:mAvgVar   11:mAvgVarStoch   12:mAvgVarR   13:t" << endl;
        int j;
        
        int nbBins = m_psdPtr->getSize();
        
        for(j=0; j<nbBins; j++)
        {
            double mAvgj = m_psdPtr->getmAvg(j);
            double npj = m_psdPtr->getNp(j);
            double Nvj = m_psdPtr->getNv(j);
            double ndmj = m_psdPtr->calculateNm(j);  // dNv / dm  ( or dN / dlog(m)  )
            double ndmjStoch = m_psdPtr->calculateNmStochastic(j);
            
            Bin* binPtr = m_psdPtr->getBinPtr(j);
            double NvR = binPtr->getNvResidual();
            
            double mInfj = m_psdPtr->getmInf(j);
            double mSupj = m_psdPtr->getmSup(j);
            double deltamj = mSupj - mInfj;
            
            double mAvgVar = m_psdPtr->getmAvgVar(j);
            double mAvgVarStoch = m_psdPtr->getmAvgVarStoch(j);
            double mAvgVarR = m_psdPtr->getmAvgVarR(j);
            
            
            stream1 << mAvgj << "   " << npj << "   " << Nvj << "   " << ndmj << "   " << ndmjStoch << "   " << NvR << "   " << deltamj << "   " << mInfj << "   " << mSupj << "   " << mAvgVar << "   " << mAvgVarStoch << "   " << mAvgVarR << "   " << t << endl;
        }
        stream1 << endl;
    }
    else
    {
        cout << "ERROR: Impossible to open the file." << endl;
    }
    
}

void NpSolver::writePsdAnalyticalSubIt(double t2)
{
    // #iteration ..  t = ..  x = ..  NvT = ... column labels:
    // 1:mAvg [g]   2:dAvg [nm]   3:np [stochastic part]   4:Nv [part/cm3]   5:nd = dN/dlogd = Nv / (logdSup - logdInf)   6:nd(d>3) / Nvtot(d>3)
    
    string pathProject = m_pathProject;
    string pathTarget = m_pathTargetPsd;
    
    //double t = getTime();
    int it = m_itCurrent;
    double NvTot = m_psdPtr->getNvT();
    double npTot = m_psdPtr->getNpT();
    
    string finalPath = pathProject.append(pathTarget);
    
    
    double time = t2;
    stringstream ss;
    ss << time;
    string strt = ss.str();
    
    
    finalPath = finalPath.append(strt);
    string dat = "_subIt.dat";
    finalPath = finalPath.append(dat);
    
    ofstream stream1(finalPath.c_str());
    if(stream1) // error test
    {
        stream1 << "#PSD File. It = " << it << "   t = " << t2  << "   NvT = " << NvTot << "   NpT = " << npTot << "   column labels: 1:mAvg [g]   2:np [stochastic part]   3:Nv [part/cm3]   4:ndm = dN/dm = Nv / (mSup - mInf) [part/cm3/size]   5:ndmStoch (ndm only by stochastic particles, without residual   6:NvR [part/cm3] (residual only)   7:deltamCell [g]   8:mInfCell [g]   9:mSupCell [g]   10:mAvgVar   11:mAvgVarStoch   12:mAvgVarR   13:t"<<endl;
        int j;
        
        int nbBins = m_psdPtr->getSize();
        
        for(j=0; j<nbBins; j++)
        {
            double mAvgj = m_psdPtr->getmAvg(j);
            double npj = m_psdPtr->getNp(j);
            double Nvj = m_psdPtr->getNv(j);
            double ndmj = m_psdPtr->calculateNm(j);  // dNv / dm  ( or dN / dlog(m)  )
            double ndmjStoch = m_psdPtr->calculateNmStochastic(j);
            
            Bin* binPtr = m_psdPtr->getBinPtr(j);
            double NvR = binPtr->getNvResidual();
            
            double mInfj = m_psdPtr->getmInf(j);
            double mSupj = m_psdPtr->getmSup(j);
            double deltamj = mSupj - mInfj;
            
            double mAvgVar = m_psdPtr->getmAvgVar(j);
            double mAvgVarStoch = m_psdPtr->getmAvgVarStoch(j);
            double mAvgVarR = m_psdPtr->getmAvgVarR(j);
            
            stream1 << mAvgj << "   " << npj << "   " << Nvj << "   " << ndmj << "   " << ndmjStoch << "   " << NvR << "   " << deltamj << "   " << mInfj << "   " << mSupj << "   " << mAvgVar << "   " << mAvgVarStoch << "   " << mAvgVarR  << "   " << t2 << endl;
        }
        stream1 << endl;
    }
    else
    {
        cout << "ERROR: Impossible to open the file." << endl;
    }
    
}

void NpSolver::writePsdClustersAnalytical(int partPerCluster)
{
    vector<vector<double> > clustersVec = getClustersVector(partPerCluster);
    
    string pathProject = m_pathProject;
    string pathTarget = m_pathTargetPsd;
    
    double t = getTime();
    int it = m_itCurrent;
    double NvTot = m_psdPtr->getNvT();
    double npTot = m_psdPtr->getNpT();
    
    string finalPath = pathProject.append(pathTarget);
    finalPath = finalPath.append("Clusters/");
    
    string fileCluster = "psdClusters";
    finalPath.append(fileCluster);
    
    double time = t;
    stringstream ss;
    ss << time;
    string strt = ss.str();
    
    
    finalPath = finalPath.append(strt);
    string dat = ".dat";
    finalPath = finalPath.append(dat);
    
    
    ofstream stream1(finalPath.c_str());
    if(stream1) // error test
    {
        stream1 << "#PSD File. It = " << it << "   t = " << t  << "   NvT = " << NvTot << "   NpT = " << npTot << "partPerCluster = " << partPerCluster <<"   column labels: 1:mAvgCluster [g]   2:ndmCluster = dN/dm = Nv / (mSup - mInf) [part/cm3/size]   3:deltamCluster   4:t"<<endl;
        
        int j;
        for(j=0; j<clustersVec.size(); j++)
        {
            double mAvgj = clustersVec[j][0];
            double ndmj = clustersVec[j][1];  // dNv / dm  ( or dN / dlog(m)  )
            double deltamj = clustersVec[j][2];
            
            stream1 << mAvgj << "   " << ndmj << "   " << deltamj << "   " << t << endl;
        }
        stream1 << endl;
    }
    else
    {
        cout << "ERROR: Impossible to open the file." << endl;
    }
}

void NpSolver::writePsdClustersAnalyticalSubIt(int partPerCluster, double t2)
{
    vector<vector<double> > clustersVec = getClustersVector(partPerCluster);
    
    string pathProject = m_pathProject;
    string pathTarget = m_pathTargetPsd;
    
    //double t = getTime();
    int it = m_itCurrent;
    double NvTot = m_psdPtr->getNvT();
    double npTot = m_psdPtr->getNpT();
    
    string finalPath = pathProject.append(pathTarget);
    finalPath = finalPath.append("Clusters/");
    
    string fileCluster = "psdClusters";
    finalPath.append(fileCluster);
    
    double time = t2;
    stringstream ss;
    ss << time;
    string strt = ss.str();
    
    
    finalPath = finalPath.append(strt);
    string dat = "_subIt.dat";
    finalPath = finalPath.append(dat);
    
    ofstream stream1(finalPath.c_str());
    if(stream1) // error test
    {
        stream1 << "#PSD File. It = " << it << "   t = " << t2  << "   NvT = " << NvTot << "   NpT = " << npTot << "partPerCluster = " << partPerCluster <<"   column labels: 1:mAvgCluster [g]   2:ndmCluster = dN/dm = Nv / (mSup - mInf) [part/cm3/size]   3:deltamCluster   4:t"<<endl;
        
        int j;
        for(j=0; j<clustersVec.size(); j++)
        {
            double mAvgj = clustersVec[j][0];
            double ndmj = clustersVec[j][1];  // dNv / dm  ( or dN / dlog(m)  )
            double deltamj = clustersVec[j][2];
            
            stream1 << mAvgj << "   " << ndmj << "   " << deltamj  << "   " << t2  << endl;
        }
        stream1 << endl;
    }
    else
    {
        cout << "ERROR: Impossible to open the file." << endl;
    }
}


void NpSolver::writePsdClustersHomogeneousGasPhase(int partPerCluster)
{
    vector<vector<double> > clustersVec = getClustersVectorHomogeneous(partPerCluster);
    
    string pathProject = m_pathProject;
    string pathTarget = m_pathTargetPsd;
    
    double t = getTime();
    int it = m_itCurrent;
    double NvTot = m_psdPtr->getNvT();
    double npTot = m_psdPtr->getNpT();
    
    string finalPath = pathProject.append(pathTarget);
    finalPath = finalPath.append("Clusters/");
    
    string fileCluster = "psdClusters";
    finalPath.append(fileCluster);
    
    double time = t;
    stringstream ss;
    ss << time;
    string strt = ss.str();
    
    
    finalPath = finalPath.append(strt);
    string dat = ".dat";
    finalPath = finalPath.append(dat);
    
    
    ofstream stream1(finalPath.c_str());
    if(stream1) // error test
    {
        stream1 << "#PSD File. It = " << it << "   t = " << t  << "   NvT = " << NvTot << "   NpT = " << npTot << "partPerCluster = " << partPerCluster <<"   column labels: 1:mAvgCluster [g]   2:nmCluster = dN/dm = Nv / (mSup - mInf) [part/cm3/size]   3:deltamCluster   4:t   5:dAvgCluster   6:ndCluster = dN/dlogd = Nv / (logdSup - logdInf) [part/cm3/size] "<<endl;
        
        int j;
        for(j=0; j<clustersVec.size(); j++)
        {
            double mAvgj = clustersVec[j][0];
            double nmj = clustersVec[j][1];  // dNv / dm  ( or dN / dlog(m)  )
            double deltamj = clustersVec[j][2];
            double dAvgCluster = clustersVec[j][3];
            double ndCluster = clustersVec[j][4];
            
            stream1 << mAvgj << "   " << nmj << "   " << deltamj << "   " << t << "   " << dAvgCluster << "   " << ndCluster << endl;
        }
        stream1 << endl;
    }
    else
    {
        cout << "ERROR: Impossible to open the file." << endl;
    }
}


void NpSolver::writePsdClustersHomogeneousGasPhaseSubIt(int partPerCluster, double t2)
{
    vector<vector<double> > clustersVec = getClustersVectorHomogeneous(partPerCluster);
    
    string pathProject = m_pathProject;
    string pathTarget = m_pathTargetPsd;
    
    int it = m_itCurrent;
    double NvTot = m_psdPtr->getNvT();
    double npTot = m_psdPtr->getNpT();
    
    string finalPath = pathProject.append(pathTarget);
    finalPath = finalPath.append("Clusters/");
    
    string fileCluster = "psdClusters";
    finalPath.append(fileCluster);
    
    double time = t2;
    stringstream ss;
    ss << time;
    string strt = ss.str();
    
    
    finalPath = finalPath.append(strt);
    string dat = ".dat";
    finalPath = finalPath.append(dat);
    
    
    ofstream stream1(finalPath.c_str());
    if(stream1) // error test
    {
        stream1 << "#PSD File. It = " << it << "   t = " << t2  << "   NvT = " << NvTot << "   NpT = " << npTot << "partPerCluster = " << partPerCluster <<"   column labels: 1:mAvgCluster [g]   2:nmCluster = dN/dm = Nv / (mSup - mInf) [part/cm3/size]   3:deltamCluster   4:t   5:dAvgCluster   6:ndCluster = dN/dlogd = Nv / (logdSup - logdInf) [part/cm3/size] "<<endl;
        
        int j;
        for(j=0; j<clustersVec.size(); j++)
        {
            double mAvgj = clustersVec[j][0];
            double nmj = clustersVec[j][1];  // dNv / dm  ( or dN / dlog(m)  )
            double deltamj = clustersVec[j][2];
            double dAvgCluster = clustersVec[j][3];
            double ndCluster = clustersVec[j][4];
            
            stream1 << mAvgj << "   " << nmj << "   " << deltamj << "   " << t2 << "   " << dAvgCluster << "   " << ndCluster << endl;
        }
        stream1 << endl;
    }
    else
    {
        cout << "ERROR: Impossible to open the file." << endl;
    }
}


void NpSolver::writeIntegralLineHomogeneousGasPhase()
{
    string pathProject = m_pathProject;
    string pathTarget = m_pathTargetGlobal;
    string finalPath = pathProject.append(pathTarget);
    string dat = ".dat";
    finalPath = finalPath.append(dat);
    ofstream stream1(finalPath.c_str(), ios::app);
    if(stream1) // error test
    {
        HomogeneousGasPhase* gasPhasePtr = m_sourcePtr->getGasPhasePtr();
        double t = getTime();
        int it = m_itCurrent;
        
        double x(1.0);
        double xshifted(1.0);
        double rhogramm = 1.0;
        double YA4(0.0);
        double wmA4(0.0);
        
        x = gasPhasePtr->getx(t);
        xshifted = x + 0.0025;
        rhogramm = gasPhasePtr->getrho(t) * 0.001;
        YA4 = gasPhasePtr->getYA4(t);
        wmA4 = gasPhasePtr->getwmA4(t); // [g/cm3/s]
        
        double NvT = m_psdPtr->getNvT();
        double NvTdetected = m_psdPtr->calculateNvTotDetected();
        
        double dNv_diameter(0.0);
        int nbBins = m_psdPtr->getSize();
        int j(0);
        for(j=0; j<nbBins; j++)
        {
            double dAvgj = m_psdPtr->getdAvgNano(j);
            double Nvj = m_psdPtr->getNv(j);
            dNv_diameter += dAvgj * Nvj;
        }
        double dAvgPsd = dNv_diameter / NvT;
        
        double sumNv(0.0);
        double medianThreshold = NvT / 2.0;
        
        double dMedian(1.0);
        
        j=0;
        while (sumNv<medianThreshold)
        {
            dMedian = m_psdPtr->getdAvgNano(j);
            double Nvj = m_psdPtr->getNv(j);
            sumNv += Nvj;
            j++;
        }
        
        double sumNvdetected(0.0);
        double dMedianDetected(1.0);
        double medianThresholdDetected = NvTdetected / 2.0;
        
        double diDetected = m_psdPtr->getSmallestDetectedDiameter();
        j=0;
        while (sumNvdetected<medianThresholdDetected)
        {
            dMedianDetected = m_psdPtr->getdAvgNano(j);
            double Nvj;
            if(dMedianDetected < diDetected)
            {
                Nvj = 0;
            }
            else
            {
                Nvj = m_psdPtr->getNv(j);
            }
            sumNvdetected+=Nvj;
            j++;
        }
        
        double sootMassConcentration(0.0);
        for(j=0; j<nbBins; j++)
        {
            double mAvgj = m_psdPtr->getmAvg(j);   // [g/part]
            double Nvj = m_psdPtr->getNv(j);       // [part/cm3]
            sootMassConcentration += mAvgj * Nvj;  // [g/cm3]
        }
        
        j=0;
        
        double sootMassConcDetected(0.0);
        for(j=0; j<nbBins; j++)
        {
            double dj = m_psdPtr->getdAvgNano(j);
            double Nvj;
            if(dj < diDetected)
            {
                Nvj = 0;
            }
            else
            {
                Nvj = m_psdPtr->getNv(j);
            }
            
            double mAvgj = m_psdPtr->getmAvg(j);   // [g/part]
            
            sootMassConcDetected += mAvgj * Nvj;  // [g/cm3]
        }
        
        
        double Ys = sootMassConcentration / rhogramm;
        double fv = Ys * rhogramm / rhoSoot;
        
        double YsDetected = sootMassConcDetected /rhogramm;
        double fvDetected = YsDetected * rhogramm / rhoSoot;
        
        double dotAt = partAggloSourceTot(t);
        double dotH = m_sourcePtr->partNuclSource(t);
        
        double YA4corr = m_sourcePtr->YA4corrected(t);
        double YA4hybrid = m_sourcePtr->YA4hybrid(t);
        double consoTot = m_sourcePtr->getConsoTotPAH();
        double avgAlphaHaca = m_sourcePtr->avgalphaHaca(t);
        
        double cmassVar = 12.0 * 1.6605e-24;
        double mPAH = 16.0 * cmassVar;  // [g]
        
        double dotPAHnucl = m_sourcePtr->getDotmPAHnucl() / mPAH;   // [molec/cm3/s]
        double dotCondTot = m_sourcePtr->partCondSourceTot(t);    // [molec/cm3/s]
        double CmA4 = m_sourcePtr->getCmA4();
        
        double dotA4gas = wmA4 / mPAH;  // [molec/cm3/s]
        
        stream1 << it << "   " << t << "   " << x << "   " << NvT << "   " << NvTdetected << "   " << dAvgPsd << "   " << dMedian << "   " << Ys << "   " << fv << "   " << dotAt << "   " << dotH << "   " << YA4 << "   " << YA4corr << "   " << consoTot << "   " << avgAlphaHaca << "   " << dMedianDetected << "   " << fvDetected << "   " << YA4hybrid << "   " << xshifted << "   " << dotPAHnucl << "   " << dotCondTot << "   " << rhogramm << "   "  << CmA4 << "   " << dotA4gas << endl;
        
    }
    else
    {
        cout << "ERROR: Impossible to open the file." << endl;
    }
}


void NpSolver::correctionYA4()
{
    double t = getTime();
    
    //Summing over time mass [g/cm3] of pyrene consumed by dimerization. used to correct YA4 to YA4'
    
    // for "consoTot" correctionType
    m_sourcePtr->addConsoTotPAH(m_timePerIt, t);
    
    // for "PAHsources" correctionType
    double wmA4 = m_sourcePtr->getDotCmA4correctedSource(t);   // in [g/cm3/s]
    double deltamA4it = wmA4 * m_timePerIt;  // in [g/cm3]
    m_sourcePtr->advanceCmA4correctedSource(deltamA4it);
    
}


void NpSolver::setSolverType(string solverType)
{
    m_solverType = solverType;
}

double NpSolver::getTimePerIt()
{
    return m_timePerIt;
}

vector<vector<double> > NpSolver::getClustersVector(int partPerCluster)
{
    vector<GasParticle*> rankedPartPtr = m_aeroSolPtr->getRankedPartPtr();
    double NvT = m_psdPtr->getNvT();
    int NpTot = m_psdPtr->getNpT();
    double NvCluster = NvT * partPerCluster / double(NpTot);
    
    
    vector<vector<double> > clustersVector; // column [0]: mAvg  column [1]: nmStoch
    
    int i(0);
    
    while(i<(NpTot-(partPerCluster-1)))
    {
        int j(0);
        double mAvgCluster(0.0);
        double mInfCluster = rankedPartPtr[i]->getMass();
        
        for(j=0; j<partPerCluster; j++)
        {
            double mAvgi = rankedPartPtr[i]->getMass();
            
            mAvgCluster += 1.0 / double(partPerCluster) * mAvgi;
            
            i++;
        }
        
        double mSupCluster(0.0);
        if(i<NpTot)
        {
            mSupCluster = rankedPartPtr[i]->getMass(); // mSup taken as the mass of the first particle of next cluster
        }
        else
        {
            mSupCluster = rankedPartPtr[i-1]->getMass(); // in the case of the last cluster, mSup is the mass of the last particle of the cluster.
        }
        
        
        double nmCluster = NvCluster / (mSupCluster - mInfCluster);
        double deltamCluster = (mSupCluster - mInfCluster);
        
        vector<double> cluster;
        cluster.push_back(mAvgCluster);
        cluster.push_back(nmCluster);
        cluster.push_back(deltamCluster);
        
        clustersVector.push_back(cluster);
    }
    
    return clustersVector;
}


vector<vector<double> > NpSolver::getClustersVectorHomogeneous(int partPerCluster)
{
    vector<GasParticle*> rankedPartPtr = m_aeroSolPtr->getRankedPartPtr();
    double NvT = m_psdPtr->getNvT();
    int NpTot = m_psdPtr->getNpT();
    double NvCluster = NvT * partPerCluster / double(NpTot);
    
    
    vector<vector<double> > clustersVector;
    // 1:mAvgCluster [g]   2:nmCluster = dN/dm = Nv / (mSup - mInf) [part/cm3/size]   3:deltamCluster   4:dAvgCluster   5:ndCluster = dN/dlogd = Nv / (logdSup - logdInf) [part/cm3/size]
    
    int i(0);
    
    while(i<(NpTot-(partPerCluster-1)))
    {
        int j(0);
        double mAvgCluster(0.0);
        double mInfCluster = rankedPartPtr[i]->getMass();
        
        for(j=0; j<partPerCluster; j++)
        {
            double mAvgi = rankedPartPtr[i]->getMass();
            
            mAvgCluster += 1.0 / double(partPerCluster) * mAvgi;
            
            i++;
        }
        
        double mSupCluster(0.0);
        if(i<NpTot)
        {
            mSupCluster = rankedPartPtr[i]->getMass(); // mSup taken as the mass of the first particle of next cluster
        }
        else
        {
            mSupCluster = rankedPartPtr[i-1]->getMass(); // in the case of the last cluster, mSup is the mass of the last particle of the cluster.
        }
        
        
        double nmCluster = NvCluster / (mSupCluster - mInfCluster);
        double deltamCluster = (mSupCluster - mInfCluster);
        
        double dSupNano = getDfromMassNano(mSupCluster);
        double dInfNano = getDfromMassNano(mInfCluster);
        
        double dAvgNano = getDfromMassNano(mAvgCluster);  // in nanometers
        
        double dlogInf = log10(dInfNano);
        double dlogSup = log10(dSupNano);
        
        double nd = NvCluster / (dlogSup - dlogInf);
        
        
        vector<double> cluster;
        cluster.push_back(mAvgCluster);
        cluster.push_back(nmCluster);
        cluster.push_back(deltamCluster);
        cluster.push_back(dAvgNano);
        cluster.push_back(nd);
        
        clustersVector.push_back(cluster);
    }
    
    return clustersVector;
}



vector<double> NpSolver::additionalResidualPerBin()
{
    vector<double> additionalResidual;
    int nBins = m_psdPtr->getSize();
    int NpTot = m_psdPtr->getNpT();
    double NvT = m_psdPtr->getNvT();
    double NvPerPart = NvT / double(NpTot);
    int i(0);
    for(i=0; i<nBins; i++)
    {
        int npl = m_psdPtr->getNp(i);
        double additionalNvR(0.0);
        
        if(npl < m_hybridTrigger && npl > 0)
        {
            additionalNvR = npl * NvPerPart;
        }
        
        additionalResidual.push_back(additionalNvR);
    }
    
    return additionalResidual;
}


void NpSolver::addResidualUnderTrigger(vector<double> additionalResidual)
{
    int nBins = m_psdPtr->getSize();
    int i(0);
    for(i=0; i<nBins; i++)
    {
        Bin* binPtr = m_psdPtr->getBinPtr(i);
        binPtr->addNvResidual(additionalResidual[i]);
        
    }
}

void NpSolver::substractResidualUnderTrigger(vector<double> additionalResidual)
{
    int nBins = m_psdPtr->getSize();
    int i(0);
    for(i=0; i<nBins; i++)
    {
        Bin* binPtr = m_psdPtr->getBinPtr(i);
        binPtr->addNvResidual(- additionalResidual[i]);
        
    }
}

double NpSolver::dotNv2pointUpwind(int i)
{
    double dotNv(0.0);
    
    double mSup = m_psdPtr->getmSup(i);
    double mInf = m_psdPtr->getmInf(i);
    
    //double mAvg = m_psdPtr->getmAvg(i);
    double mAvgVarR = m_psdPtr->getmAvgVarR(i);
    //cout << "mAvgVarR(" << i << ") = " << mAvgVarR << endl;
    
    //double G = m_growthParkPtr->g(i);
    double t = getTime();
    double G = m_sourcePtr->deltamTotGrowthVector(t)[i];
    
    double Nv = m_psdPtr->getNv(i);
    double nBins = m_psdPtr->getSize();
    
    if(G >=0)
    {
        if(i == 0)
        {
            //double mAvgPlusOne = m_psdPtr->getmAvg(i+1);
            double mAvgVarRPlusOne = m_psdPtr->getmAvgVarR(i+1);
            
            if(m_sectionalMethod == "2pointUpwindmAvg")
            {
                //dotNv = - (G * Nv) / (mAvgPlusOne - mAvg);
                dotNv = - (G * Nv) / (mAvgVarRPlusOne - mAvgVarR);
            }
            else if(m_sectionalMethod == "2pointUpwindmSup")
            {
                dotNv = - (G * Nv) / (mSup - mInf);
            }
            else
            {
                cout << "ERROR ::dotNv2pointUpwind m_sectionalMethod" << endl;
            }
            
        }
        else if(i == nBins-1)
        {
            double mSupMinusOne = m_psdPtr->getmSup(i-1);
            double mInfMinusOne = m_psdPtr->getmInf(i-1);
            
            //double mAvgMinusOne = m_psdPtr->getmAvg(i-1);
            double mAvgVarRMinusOne = m_psdPtr->getmAvgVarR(i-1);
            
            //double GMinusOne = m_growthParkPtr->g(i-1);
            double GMinusOne = m_sourcePtr->deltamTotGrowthVector(t)[i-1];
            
            double NvMinusOne = m_psdPtr->getNv(i-1);
            
            if(m_sectionalMethod == "2pointUpwindmAvg")
            {
                //dotNv = (GMinusOne * NvMinusOne) / (mAvg - mAvgMinusOne);
                dotNv = (GMinusOne * NvMinusOne) / (mAvgVarR - mAvgVarRMinusOne);
            }
            else if(m_sectionalMethod == "2pointUpwindmSup")
            {
                dotNv = (GMinusOne * NvMinusOne) / (mSupMinusOne - mInfMinusOne);
                
            }
            else
            {
                cout << "ERROR ::dotNv2pointUpwind m_sectionalMethod" << endl;
            }
            
        }
        else
        {
            double mSupMinusOne = m_psdPtr->getmSup(i-1);
            double mInfMinusOne = m_psdPtr->getmInf(i-1);
            
            //double mAvgMinusOne = m_psdPtr->getmAvg(i-1);
            //double mAvgPlusOne = m_psdPtr->getmAvg(i+1);
            double mAvgVarRMinusOne = m_psdPtr->getmAvgVarR(i-1);
            double mAvgVarRPlusOne = m_psdPtr->getmAvgVarR(i+1);
            
            //double GMinusOne = m_growthParkPtr->g(i-1);
            double GMinusOne = m_sourcePtr->deltamTotGrowthVector(t)[i-1];
            
            double NvMinusOne = m_psdPtr->getNv(i-1);
            
            if(m_sectionalMethod == "2pointUpwindmAvg")
            {
                //dotNv = (GMinusOne * NvMinusOne) / (mAvg - mAvgMinusOne) - (G * Nv) / (mAvgPlusOne - mAvg);
                dotNv = (GMinusOne * NvMinusOne) / (mAvgVarR - mAvgVarRMinusOne) - (G * Nv) / (mAvgVarRPlusOne - mAvgVarR);
            }
            else if(m_sectionalMethod == "2pointUpwindmSup")
            {
                dotNv = (GMinusOne * NvMinusOne) / (mSupMinusOne - mInfMinusOne) - (G * Nv) / (mSup - mInf);

            }
            else
            {
                cout << "ERROR ::dotNv2pointUpwind m_sectionalMethod" << endl;
            }

            
        }
        
    }
    else // G < 0
    {
        if(i == 0)
        {
            //double mAvgPlusOne = m_psdPtr->getmAvg(i+1);
            double mAvgVarRPlusOne = m_psdPtr->getmAvgVarR(i+1);
            
            //double GPlusOne = m_growthParkPtr->g(i+1);
            double GPlusOne = m_sourcePtr->deltamTotGrowthVector(t)[i+1];
            
            double NvPlusOne = m_psdPtr->getNv(i+1);
            
            //dotNv =  - (GPlusOne * NvPlusOne) / (mAvgPlusOne - mAvg);
            dotNv =  - (GPlusOne * NvPlusOne) / (mAvgVarRPlusOne - mAvgVarR);
        }
        else if(i == nBins-1)
        {
            //double mAvgMinusOne = m_psdPtr->getmAvg(i-1);
            double mAvgVarRMinusOne = m_psdPtr->getmAvgVarR(i-1);
            
            //dotNv = (G * Nv) / (mAvg - mAvgMinusOne);
            dotNv = (G * Nv) / (mAvgVarR - mAvgVarRMinusOne);
        }
        else
        {
            //double mAvgMinusOne = m_psdPtr->getmAvg(i-1);
            //double mAvgPlusOne = m_psdPtr->getmAvg(i+1);
            double mAvgVarRMinusOne = m_psdPtr->getmAvgVarR(i-1);
            double mAvgVarRPlusOne = m_psdPtr->getmAvgVarR(i+1);
            
            //double GPlusOne = m_growthParkPtr->g(i+1);
            double GPlusOne = m_sourcePtr->deltamTotGrowthVector(t)[i+1];
            
            double NvPlusOne = m_psdPtr->getNv(i+1);
            
            //dotNv = (G * Nv) / (mAvg - mAvgMinusOne) - (GPlusOne * NvPlusOne) / (mAvgPlusOne - mAvg);
            dotNv = (G * Nv) / (mAvgVarR - mAvgVarRMinusOne) - (GPlusOne * NvPlusOne) / (mAvgVarRPlusOne - mAvgVarR);
        }
    }
    
    return dotNv;
}

void NpSolver::reequilibrate_mAvgVar(int i)
{
    Bin* binPtr = m_psdPtr->getBinPtr(i);
    double mInf = binPtr->getmInf();
    double mSup = binPtr->getmSup();
    double mAvgVar = binPtr->getmAvgVar();
    double mAvgVarStoch = binPtr->getmAvgVarStoch();
    double mAvgVarR = binPtr->getmAvgVarStoch();
    int np = binPtr->getNp();
    np = double(np);
    int NpTot = m_psdPtr->getNpT();
    NpTot = double(NpTot);
    double NvT = m_psdPtr->getNvT();
    double NvStoch = double(np)/double(NpTot)*NvT;
    double NvR = binPtr->getNvResidual();
    
    if(mAvgVar < mInf)
    {
        // in this case, transfer to bin n-1
        
        // case i = 0
        if(i == 0)
        {
            // !! to write !!
            // mass loss? oxidation?
        } else {
            // general case
            Bin* binMinusPtr = m_psdPtr->getBinPtr(i-1);
            double mAvgVarRminus = binMinusPtr->getmAvgVarR();
            double mInfMinus = binMinusPtr->getmInf();
            double mSupMinus = binMinusPtr->getmSup();
            if(mAvgVarRminus >= mInfMinus && mAvgVarRminus <= mSupMinus)
            {
                // Nv1 = (mTotBin - mAvg2*NvTotBin) / (mAvg1 - mAvg2)
                // Nv2 = NvTotBin - Nv1
                // here Nv1 is NvStoch + NvR after reequilibrate
                // NvTot is NvStoch + NvR before reequilibrate
                // Nv2 is the amont of particles transfered to neighboring bin
                // mAvg1 corresponds to mAvgVarStoch (target 1)
                // mAvg2 corresponds to mAvgVarR_neighbor (target 2)
                
                // therefore, NvR(after reequilibrate) = Nv1 - NvStoch
                // NvR_neighbour(after reequilibrate) = NvR_neighbour(before reequilibrate) + Nv2
                
                double mTotBin = mAvgVarR * NvR + mAvgVarStoch * NvStoch;
                double NvTotBin = NvR + NvStoch;
                double mAvg1 = mAvgVarStoch;
                double mAvg2 = mAvgVarRminus;
                
                double Nv1 = (mTotBin - mAvg2*NvTotBin) / (mAvg1 - mAvg2);
                double Nv2 = NvTotBin - Nv1;
                
                // updating neighboring bin
                double NvRminusPrevious = binMinusPtr->getNvResidual();
                double NvRminusUpdated = NvRminusPrevious + Nv2;
                binMinusPtr->setNvResidual(NvRminusUpdated);
                
                // updating bin n
                binPtr->setmAvgVarR(mAvgVarStoch);
                binPtr->setNvResidual(Nv1 - NvStoch);
                
                
            } else {
                cout << "ERROR reequilibrate mAvgVar: mAvgVar_iMinusOne out of section" << endl;
            }
            
            
        }
        
        
    } else if(mAvgVar > mSup)
    {
        int nBins = m_psdPtr->getSize();
        if(i == (nBins - 1))
        {
            // case last bin, to write !!
        } else{
            // in this case, transfer to bin n+1
            Bin* binPlusPtr = m_psdPtr->getBinPtr(i+1);
            double mAvgVarRplus = binPlusPtr->getmAvgVarR();
            double mInfPlus = binPlusPtr->getmInf();
            double mSupPlus = binPlusPtr->getmSup();
            
            if(mAvgVarRplus >= mInfPlus && mAvgVarRplus <= mSupPlus)
            {
                // Nv1 = (mTotBin - mAvg2*NvTotBin) / (mAvg1 - mAvg2)
                // Nv2 = NvTotBin - Nv1
                // here Nv1 is NvStoch + NvR after reequilibrate
                // NvTot is NvStoch + NvR before reequilibrate
                // Nv2 is the amont of particles transfered to neighboring bin
                // mAvg1 corresponds to mAvgVarStoch (target 1)
                // mAvg2 corresponds to mAvgVarR_neighbor (target 2)
                
                // therefore, NvR(after reequilibrate) = Nv1 - NvStoch
                // NvR_neighbour(after reequilibrate) = NvR_neighbour(before reequilibrate) + Nv2
                
                double mTotBin = mAvgVarR * NvR + mAvgVarStoch * NvStoch;
                double NvTotBin = NvR + NvStoch;
                double mAvg1 = mAvgVarStoch;
                double mAvg2 = mAvgVarRplus;
                
                double Nv1 = (mTotBin - mAvg2*NvTotBin) / (mAvg1 - mAvg2);
                double Nv2 = NvTotBin - Nv1;
                
                // updating neighboring bin
                double NvRplusPrevious = binPlusPtr->getNvResidual();
                double NvRplusUpdated = NvRplusPrevious + Nv2;
                binPlusPtr->setNvResidual(NvRplusUpdated);
                
                // updating bin n
                binPtr->setmAvgVarR(mAvgVarStoch);
                binPtr->setNvResidual(Nv1 - NvStoch);
                
                
            } else {
                cout << "ERROR reequilibrate mAvgVar: mAvgVar_iMinusOne out of section" << endl;
            }
            
        }
        
        
    } else
    {}
    
}

void NpSolver::check_reequilibrate_all_bins()
{
    int nBins = m_psdPtr->getSize();
    int i(0);
    for(i=0; i<nBins; i++)
    {
        double mInf = m_psdPtr->getmInf(i);
        double mSup = m_psdPtr->getmSup(i);
        double mAvgVar = m_psdPtr->getmAvgVar(i);
        if(mAvgVar < mInf | mAvgVar > mSup)
        {
            reequilibrate_mAvgVar(i);
        }
    }
}

double NpSolver::deltaNvR_from_residualMassCorrection(int bin, vector<double> residualMassCorrection){
    double deltaNvR(0.0);
    
    int i = bin;
    int nBins = m_psdPtr->getSize();
    
    if (i == 0) {
        //double residualMassMinusOne = residualMassCorrection[i - 1];
        double residualMass = residualMassCorrection[i];
        double residualMassPlusOne = residualMassCorrection[i + 1];
        
        //double mAvgVarMinusOne = m_psdPtr->getmAvgVar(i - 1);
        double mAvgVar = m_psdPtr->getmAvgVar(i);
        double mAvgVarPlusOne = m_psdPtr->getmAvgVar(i + 1);
        
        
        double deltaNvR_out(0.0);
        if (residualMass > 0.0) {  // transfer to the left or transfer to the right
            deltaNvR_out = - residualMass / (mAvgVarPlusOne - mAvgVar);
        } else {
            //deltaNvR_out = - residualMass / (mAvgVarMinusOne - mAvgVar);
            //cout << "could not apply correction mass from 0th section: " << residualMass << endl << endl;
            deltaNvR_out = 0.0;
        }
        
        double deltaNvR_from_plusOne = residualMassPlusOne / (mAvgVar - mAvgVarPlusOne);
        if (deltaNvR_from_plusOne < 0.0){
            deltaNvR_from_plusOne = 0.0;
        }
        
        deltaNvR = deltaNvR_out + deltaNvR_from_plusOne;
        
        //cout << endl << "i = " << i << "   deltaNvR_out = " << deltaNvR_out << "   deltaNvR_from_plusOne = " << deltaNvR_from_plusOne << endl << endl;
        
    } else if (i == (nBins - 1)) {
        double residualMassMinusOne = residualMassCorrection[i - 1];
        double residualMass = residualMassCorrection[i];
        //double residualMassPlusOne = residualMassCorrection[i + 1];
        
        double mAvgVarMinusOne = m_psdPtr->getmAvgVar(i - 1);
        double mAvgVar = m_psdPtr->getmAvgVar(i);
        //double mAvgVarPlusOne = m_psdPtr->getmAvgVar(i + 1);
        
        double deltaNvR_from_minusOne = residualMassMinusOne / (mAvgVar - mAvgVarMinusOne);
        if  (deltaNvR_from_minusOne < 0.0){
            deltaNvR_from_minusOne = 0.0; // taken into account only if > 0, otherwise transfer to the other neigboring cell
        }
        
        double deltaNvR_out(0.0);
        if (residualMass > 0.0) {  // transfer to the left or transfer to the right
            //deltaNvR_out = - residualMass / (mAvgVarPlusOne - mAvgVar);
            deltaNvR_out = 0.0;
            //cout << "could not apply correction mass from last section: " << residualMass << endl;
        } else {
            deltaNvR_out = - residualMass / (mAvgVarMinusOne - mAvgVar);
        }
        
        deltaNvR = deltaNvR_out + deltaNvR_from_minusOne;
        
        //cout << endl << "i = " << i << "   deltaNvR_out = " << deltaNvR_out << "   deltaNvR_from_minusOne = " << deltaNvR_from_minusOne  << endl << endl;
    } else
    {
        double residualMassMinusOne = residualMassCorrection[i - 1];
        double residualMass = residualMassCorrection[i];
        double residualMassPlusOne = residualMassCorrection[i + 1];
        
        double mAvgVarMinusOne = m_psdPtr->getmAvgVar(i - 1);
        double mAvgVar = m_psdPtr->getmAvgVar(i);
        double mAvgVarPlusOne = m_psdPtr->getmAvgVar(i + 1);
        
        double deltaNvR_from_minusOne = residualMassMinusOne / (mAvgVar - mAvgVarMinusOne);
        if  (deltaNvR_from_minusOne < 0.0){
            deltaNvR_from_minusOne = 0.0; // taken into account only if > 0, otherwise transfer to the other neigboring cell
        }
        
        double deltaNvR_out(0.0);
        if (residualMass > 0.0) {  // transfer to the left or transfer to the right
            deltaNvR_out = - residualMass / (mAvgVarPlusOne - mAvgVar);
        } else {
            deltaNvR_out = - residualMass / (mAvgVarMinusOne - mAvgVar);
        }
        
        double deltaNvR_from_plusOne = residualMassPlusOne / (mAvgVar - mAvgVarPlusOne);
        if (deltaNvR_from_plusOne < 0.0){
            deltaNvR_from_plusOne = 0.0;
        }
        
        deltaNvR = deltaNvR_out + deltaNvR_from_minusOne + deltaNvR_from_plusOne;
        //cout << endl << "i = " << i << "   deltaNvR_out = " << deltaNvR_out << "   deltaNvR_from_minusOne = " << deltaNvR_from_minusOne << "   deltaNvR_from_plusOne = " << deltaNvR_from_plusOne << endl << endl;
    }
    
    return deltaNvR;
}


double NpSolver::getDfromMassNano(double m)  // for sphere
{
    double v = m/rhoSoot;   // [cm3]  m_mAvg in [g]
    double oneThird = 1.0/3.0;
    double d3 = v*6.0/pi;
    double dcm = pow(d3, oneThird);  // [in cm]
    double dnm = dcm * 1.0e7;   // [in nm]
    return dnm;
}

