//
//  Psd.hpp
//  soot_stochastic
//
//  Created by Alexandre Bouaniche on 29/08/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#ifndef Psd_hpp
#define Psd_hpp

#include <stdio.h>
#include <vector>

#include "Bin.hpp"
#include "Grid.hpp"
#include "GridCell.hpp"
#include "AerosolPhase.hpp"

class Psd
{
public:
    //Psd(Grid initialGrid, AerosolPhase* aeroSolPtr);
    Psd(Grid initialGrid, AerosolPhase* aeroSolPtr, double initNvT);
    Psd(Grid initialGrid, AerosolPhase* aeroSolPtr, double initNvT, double smallerDetectedDiameter);
    
    // case initSectional without AerosolPhase*. All Psd information is in NvSectional of each bin. Initial nd or nm is the input
    Psd(Grid initialGrid, double n0, double x0, std::string ndType, std::string PsdInitMethod);  // ndType: "dlog", "dm"    dlog-> nd = nv / (dlogSup - dlogInf). dm -> nm = nv / (mSup - mInf).
    // PsdInitMethod: "sectional", "npResidual"
    
    // with smallestDetectedDiameter
    Psd(Grid initialGrid, double n0, double x0, std::string ndType, std::string PsdInitMethod, double smallerDetectedDiameter);
    
    
    Psd(Grid initialGrid, int bin, double NvTinit, std::string PsdInitMethod, double smallestDetectedDiameter); // same but initializing with only one bin containing NvTinit
    Psd(Grid initialGrid, int bin, double NvTinit, std::string ndType, std::string PsdInitMethod); // without smallestDetectedDiameter (for analytical)
    
    
    
    // case npResidual with AerosolPhase but also setting nvResidual to compensate small NpTot
    Psd(Grid initialGrid, AerosolPhase* aeroSolPtr, double n0, double x0, std::string ndType, std::string PsdInitMethod);
    
    // with smallestDetectedDiameter
    Psd(Grid initialGrid, AerosolPhase* aeroSolPtr, double n0, double x0, std::string ndType, std::string PsdInitMethod, double smallestDetectedDiameter);
    
    
    int getSize();
    
    double getNvT();
    double getSumNv(); // should be equal to getNvT that comes from m_nvT that is set by NpSolver by solving global equation with dotH and dotAt but can be different (problems of Number conservation?)
    double getmTot();  // to check mass evolution/conservation (by bin)
    int getNpT();
    
    Bin getBin(int i);
    Bin* getBinPtr(int i);      // returns a pointer to Bin i
    
    double getmAvg(int i);      // [mass/particle]  average mass of a particle in Bin i
    double getmInf(int i);      // [mass/particle]  min mass of a particle in Bin i
    double getmSup(int i);      // [mass/particle]  max mass of a particle in Bin i
    
    double getdAvgNano(int i);  // diameter in nanometers
    double getdInfNano(int i);
    double getdSupNano(int i);
    
    double getsAvg(int i);    // in [cm2]
    double getsInf(int i);
    double getsSup(int i);
    
    int getNp(int i);        // [part] number of stochastic particles in Bin i
    double getNv(int i);        // [part/volume]
    
    double calculateNm(int i);  // [part/volume/xcoordinate in size space] -> Number density function. Calculated from the Nv in the Psd bins. These need to be up to date. nm with xcoordinate in mass != nd with x coordinate in diameter.
    // dNv / dm  ( or dN / dlog(m)  )
    
    double calculateNd(int i);  // [part/volume/xcoordinate in size space (diameter nm)] -> Number density function. Calculated from the Nv in the Psd bins. These need to be up to date. nm with xcoordinate in mass != nd with x coordinate in diameter.
    // dNv / dlog(d)
    // !!!  Nd will be different depending on the unit of diameter used. usually nanometer used   !!!
    
    double calculateNmStochastic(int i); // same as Nm but only with stochastic particles without accounting for Nv residual
    
    double calculateMv(int i);  // [mass/volume]
    double calculateMvT();      // [mass/volume]
    
    
    // for advancing the PSD in time with s2solver
    void setNvt(double nvT);         // will be set by S2solver at each time step
    
    void countAndSetAllNp();           // counts Np in AerosolPhase and sets it in all Bins. (S2solver will reallocate particles directly modifying the AerosolPhase object)
    
    void calcAndSetAllNv();          // will be calculated and set from the npi and nvT at each time step
    
    void showPsd();
    double calculateNvTotDetected();
    double getSmallestDetectedDiameter();
    
    double getmAvgVar(int i);
    double getmAvgVarR(int i);
    double getmAvgVarStoch(int i);
    
    void setmAvgVar(int i, double mAvgVar);
    void setmAvgVar_as_mAvgConst();
    void setmAvgVar_as_mAvgVarStoch();
    void setmAvgVarR_as_mAvgConst();
    void setmAvgVarR_as_mAvgVarStoch();
    
    void calcAndSetAllmAvgVar();
    
    double calcmAvgVar(double NvStoch, double mAvgVarStoch, double NvR, double mAvgVarR, int bin, double mNucleation);
    //double calcmAvgVar(int i);
    
    double getSfromMass(double m);
    
private:
    std::vector<Bin> m_binVector;
    double m_nvT;                      //  [part/m3]  sum of Nv for all Bins
    AerosolPhase* m_aeroSolPtr;
    double m_smallestDetectedDiameter;  // [nm] for comparison to experimental
    
    static const double rhoSoot;
    static const double pi;
    
};



#endif /* Psd_hpp */
