//
//  main.cpp
//  soot_stochastic
//
//  Created by Alexandre Bouaniche on 22/06/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#include "/Users/bouaniche/nps/soot_stochastic/main.hpp"

using namespace std;

int main()
{
    
    //user inputs  *******************************************************************
    
    // Problem solved
    //string solverType = "HomogeneousGasPhase";
    string solverType = "analytical";
    
    // PBE solving
    //string pbeMethod = "npBasic";
    //string pbeMethod = "npBasicPart";
    //string pbeMethod = "npResidual";
    //string pbeMethod = "npResidualPart";
    //string pbeMethod = "sectional";
    //string pbeMethod = "hybrid";
    string pbeMethod = "hybridPart";
    
    // sectional method type (for growth)
    //string sectionalMethod = "twoPoints";
    string sectionalMethod = "park";
    //string sectionalMethod = "2pointGdt";
    //string sectionalMethod = "2pointUpwindmAvg";
    //string sectionalMethod = "2pointUpwindmSup";
    
    // Paths
    string pathProject("/Users/bouaniche/nps");
    // string pathProject("./");
    
    // Output files
    string pathTargetPsd("/samples/jaerosci/park3/fs_2/hybrid/hybridPart_park/outputs/psd/psd");
    string pathTargetGlobal("/samples/jaerosci/park3/fs_2/hybrid/hybridPart_park/outputs/global/global");
    //string pathTagetInputsHomogeneousPhase("/samples/analytical_cases/5.1all1/inputs/HomogeneousGasPhase/Kraft7/");
    
    //string pathTargetPsd("outputs/psd/psd");
    //string pathTargetGlobal("outputs/global/global");
    
    //int partPerCluster = 10;
    int partPerCluster = 1;  // !!!  set as 1 if clusters not used  !!!
    
    int hybridTrigger = 5;
    
    
    // Input files
    //string HomogeneousPhaseDataFileName("data.txt");
    //string HomogeneousPhaseLabelFileName("labels.txt");
    
    //Grid (particle mass) parameters
    // Type
    //string gridType = "standard";
    //string gridType = "geometric2";
    string gridType = "geometricFs";
    //string gridType = "regular";
    //string gridType = "exponential";
    
    // All Grids Parameters
    int nBins = 40;
    
    // for "standard" grids
    //double factorFirstCell = 0.5;
    //double c2coef = 2*12*1.6605E-24;  // for "standard" gridType
    //double mMin = 1.0;
    //double mSupMax = 1.0e5;
    double factorFirstCell = 0.0;
    double c2coef = 0.0;
    double mMin = 0.0;
    double mSupMax = 0.0;
    
    // for "geometricFs" grids or "exponential" grids
    double firstCellmAvg = 1.0; // for "geometric2" or "geometricFs" gridType
    double fs = 2.0;
    
    // for "regular" grids: geometricFs parameters + gridIncrement
    double gridIncrement = 0.0;
    
    // for "exponential" grids: fill in mMin in "standard" grid section or here
    double alphaGrid = 0.0;
    //double mMin = 2.0/3.0 * 0.1;
    
    // Initial AerosolPhase (Psd)
    string initShape = "exponential";
    //string initShape = "binInit";
    int npTot = 1.0e3;
    
    //exponential
    //double n0 = 1.0e-20;
    double n0 = 1.0e-5;
    double x0 = 1.0;
    
    // binInit
    //int binInit = 0;
    //double initNvTbin = 1.0/1.5; // initial NvT. not necessary if creator with n0 and x0 is used.
    int binInit = 1;
    //double initNvTbin = 1.0*(4.0/3.0 - 2.0/3.0);
    double initNvTbin = 1.0e-5;
    
    string initType = "mass";
    // string initType = "diameter";
    
    
    // Experimental: smallest particles detected for output
    //double smallestDetectedDiameter = 3.0; // in [nm]
    double smallestDetectedDiameter = 0.0; // in [nm]
    
    // SootSourceTerms model parameters
    
    // Type
    //string aggloType = "standard";
    //string aggloType = "harmonicAvgDc";  // Dc calculated based on fractal law
    string aggloType = "betaConst";
    
    string surfType = "sphere";
    //string surfType = "fractal";       // Surface for reaction calculated based on fractal law
    
    //string massTransferType = "aggloTensor";
    string massTransferType = "kumar96";
    //string massTransferType = "geo2SPAMM";  // only for "geometric2" gridType
    
    //string nuclType = "kraft";
    //string nuclType = "exponential";  // only for analytical test case because nucleation at all sizes
    string nuclType = "dotHconst";  //
    
    //string PAHType = "corrected";
    //string A4correctionType = "PAHsources";
    
    //string growthType = "standard"; // size/chemistry dependant
    string growthType = "G0mTheta";
    
    //string growthType = "constant";  // analytical
    //string growthType = "linear";  // analytical
    
    //string HACAType = "appel";
    //string HACAType = "constant";
    
    
    
    // source constants
    double beta0 = 0.0; // agglo
    double g0 = 1.0;  // growth
    double b0 = 1.0e0; // nucleation / exponential nucleation
    double x0n = 0.0;  // exponential nucleation
    //double alphaHaca = 1.0;
    
    // parameters for GrowthTensor
    double teta = 1.0;
    double Malpha = 1.0e-5;
    
    //aggregate shape model
    //double primaryDiameter = 5.0; //imposed primary particle diameter in nanometers.
    //double fractalDim = 1.73;  // from YON
    //double kf = 1.94;  // from Yon
    
    // TimeStep settings
    
    //string tStepType = "constant";
    //string tStepType = "fixedAlpha";
    //string tStepType = "growth";
    string tStepType = "alphaAndgSubsteps";
    
    //string growthtStepType = "growthStochastic";
    string growthtStepType = "growthSectional";
    
    double alphaAgglo = 0.01;
    //double alphaAgglo = 0.02;
    
    double maxGcoef = 0.01;
    
    // NpSolver settings
    
    // fixed time step
    int itTotal = 0;
    double timePerIt = 1.0e-2;
    
    // variable time step
    //double tFinal = 9.7;
    double tFinal = 10.39;
    
    
    int writeFreq = 1; // NpSolver will write outputs every writeFreq iteration
    int showPsdFreq = 1e2; // NpSolver will make a console print every showPsdFreq iteration
    //int showPsdFreq = 1;
    
    // writing additional writePsd, for growth subIteration within one agglo iteration. Just for a specified time range
    bool subItWrite = true;
    //bool subItWrite = false;
    double subItWrite_tmin = 9.6;
    double subItWrite_tmax = tFinal;
    
    //end user inputs  *******************************************************************
    
    
    //Objects and pointers Initialization. Solving
    
    if(solverType == "HomogeneousGasPhase")
    {
        //mainHomogeneousGasPhaseStandard(pathProject, pathTagetInputsHomogeneousPhase, HomogeneousPhaseDataFileName, HomogeneousPhaseLabelFileName, gridType, nBins, mSupMax, factorFirstCell, mMin, c2coef, n0, x0, npTot, initType, smallestDetectedDiameter, solverType, aggloType, kf, primaryDiameter, fractalDim, surfType, massTransferType, nuclType, PAHType, A4correctionType,growthType, HACAType, itTotal, timePerIt, pathTargetPsd, pathTargetGlobal);
    }
    else if(solverType == "analytical")
    {
        mainAnalyticalStandard(gridType, nBins, mSupMax, factorFirstCell, mMin, n0, x0, npTot, initType, solverType, aggloType, beta0, massTransferType, nuclType, b0, x0n, growthType, g0, itTotal, timePerIt, pathProject, pathTargetPsd, pathTargetGlobal, c2coef, firstCellmAvg, fs, initShape, binInit, initNvTbin, smallestDetectedDiameter, pbeMethod, teta, Malpha, alphaAgglo, maxGcoef, sectionalMethod, tStepType, growthtStepType, partPerCluster, hybridTrigger, tFinal, writeFreq, showPsdFreq, subItWrite, subItWrite_tmin, subItWrite_tmax, gridIncrement, alphaGrid);
        
        
    }
    else
    {
        cout << "ERROR: solverType" << endl;
    }
    
    return 0;
}
