//
//  mainStandard.cpp
//  npsSolver
//
//  Created by Alexandre Bouaniche on 11/01/2018.
//  Copyright © 2018 Alexandre Bouaniche. All rights reserved.
//

#include "mainStandard.hpp"

using namespace std;

void mainAnalyticalStandard(string gridType, int nBins, double mSupMax, double factorFirstCell, double mMin, double n0, double x0, int npTot, string initType, string solverType, string aggloType, double beta0, string massTransferType, string nuclType, double b0, double x0n, string growthType, double g0, int itTotal, double timePerIt, string pathProject, string pathTargetPsd, string pathTargetGlobal, double c2coef, double firstCellmAvg, double fs, string initShape, int binInit, double initNvTbin, double smallestDetectedDiameter, string pbeMethod, double teta, double Malpha, double alphaAgglo, double maxGcoef, string sectionalMethod, string tStepType, string growthtStepType, int partPerCluster, int hybridTrigger, double tFinal, int writeFreq, int showPsdFreq, bool subItWrite, double subItWrite_tmin, double subItWrite_tmax, double gridIncrement, double alphaGrid)
{
    // CPU time
    clock_t cpuTime;
    cpuTime = clock();
    
    
    // objects initialization
    Grid grid1 = Grid();  // grid with no parameters
    if(gridType == "standard")
    {
        grid1 = Grid(gridType, nBins, mSupMax, factorFirstCell, mMin, c2coef); // for standard
    }
    else if(gridType == "geometricFs" | gridType == "geometricFsGhost")
    {
        grid1 = Grid(nBins, firstCellmAvg, fs, gridType);
    }
    else if(gridType == "regular")
    {
        grid1 = Grid(nBins, firstCellmAvg, fs, gridType, gridIncrement);
    }
    else if(gridType == "exponential")
    {
        grid1 = Grid(nBins, firstCellmAvg, fs, gridType, gridIncrement, alphaGrid, mMin);
    }
    else
    {
        cout << "ERROR gridType initialization" << endl;
    }
    
    Grid* gridPtr = &grid1;
    grid1.showGrid();
    
    
    AerosolPhase aerosolPhase1 = AerosolPhase();  // AerosolPhase with no parameters
    if(initShape == "exponential")
    {
        aerosolPhase1 = AerosolPhase(gridPtr, n0, x0, npTot, initType);
    }
    else if(initShape == "binInit")
    {
        aerosolPhase1 = AerosolPhase(gridPtr, npTot, binInit, initNvTbin);
    }
    else
    {
        cout << "ERROR initShape AerosolPhase initialization" << endl;
    }
    AerosolPhase* aerosolPtr = &aerosolPhase1;
    
    
    Psd psd1 = Psd(grid1, aerosolPtr, initNvTbin); // Psd with no parameters
    if(pbeMethod == "npBasic" || pbeMethod == "npBasicPart")
    {
        double initialNvT = aerosolPhase1.getInitNvT();
        psd1 = Psd(grid1, aerosolPtr, initialNvT, smallestDetectedDiameter);
        psd1.countAndSetAllNp();
        psd1.calcAndSetAllNv();
        
        psd1.setmAvgVar_as_mAvgVarStoch();
        psd1.setmAvgVarR_as_mAvgVarStoch();
    }
    else if(pbeMethod == "sectional" && initShape == "exponential")
    {
        psd1 = Psd(grid1, n0, x0, "dm", "sectional");
    }
    else if(pbeMethod == "sectional" && initShape == "binInit")
    {
        double initialNvT = aerosolPhase1.getInitNvT();
        psd1 = Psd(grid1, binInit, initialNvT, "dm", "sectional");
    }
    else if( (pbeMethod == "npResidual" || pbeMethod == "npResidualPart" || pbeMethod == "hybrid" || pbeMethod == "hybridPart") && initShape == "exponential")
    {
        psd1 = Psd(grid1, aerosolPtr, n0, x0, "dm", pbeMethod);
    }
    else if( (pbeMethod == "npResidual" || pbeMethod == "npResidualPart" || pbeMethod == "hybrid" || pbeMethod == "hybridPart") && initShape == "binInit")
    {
        double initialNvT = aerosolPhase1.getInitNvT();
        psd1 = Psd(grid1, aerosolPtr, initialNvT, smallestDetectedDiameter);
        psd1.countAndSetAllNp();
        psd1.calcAndSetAllNv();
        
        psd1.setmAvgVar_as_mAvgVarStoch();
        psd1.setmAvgVarR_as_mAvgVarStoch();
    }
    else
    {
        cout << "ERROR pbeMethod for Psd initialization" << endl;
    }
    
    // in order to have initial mAvgVar values. Case npBasic or npBasicPart treated above. Case npResidual, npResidualPart, hybrid or hybridPart treated directly in Psd creator.
    if(pbeMethod == "sectional")
    {
        psd1.setmAvgVar_as_mAvgConst();
        psd1.setmAvgVarR_as_mAvgConst();
    }
    
    
    Psd* psdPtr = &psd1;
    
    AgglomerationTensor aggloTensor = AgglomerationTensor(psdPtr);
    AgglomerationTensor* aggloTensorPtr = &aggloTensor;
    
    SootSourceTerm source = SootSourceTerm(psdPtr, solverType);
    SootSourceTerm* sourcePtr = &source;
    
    source.setAggloTensor(aggloTensorPtr);
    source.setAggloType(aggloType);
    source.setbetaConst(beta0);
    
    source.setMassTransferType(massTransferType);
    
    source.setNuclType(nuclType);
    source.setb0(b0);
    source.setx0n(x0n);
    
    //source.setGrowthType(growthType);
    source.setg0(g0);
    source.setTheta(teta);
    
    
    GrowthPark growthPark1 = GrowthPark(psdPtr, sourcePtr, solverType, fs);
    GrowthPark* growthParkPtr = &growthPark1;
    
    TimeStep tstep = TimeStep(alphaAgglo, sourcePtr, psdPtr, tStepType, maxGcoef, growthtStepType, sectionalMethod, growthParkPtr);
    TimeStep* timeStepPtr = &tstep;
    
    double mass0 = psd1.getmTot();
    
    NpSolver solver = NpSolver(sourcePtr, aerosolPtr, psdPtr, itTotal, timePerIt, pathProject, pathTargetPsd, pathTargetGlobal, solverType, timeStepPtr, tFinal, pbeMethod, partPerCluster, writeFreq, sectionalMethod, growthParkPtr, hybridTrigger, showPsdFreq, subItWrite, subItWrite_tmin, subItWrite_tmax);
    
    // solve
    solver.solve();
    
    double massFinal = psd1.getmTot();
    double NvTfinal = psd1.getNvT();
    double sumNvFinal = psd1.getSumNv();
    
    // show grid for verification
    grid1.showGrid();
    
    // Check sumNv vs NvT (number conservation test)
    cout << endl << "NvT = " << NvTfinal << "   . SumNv = " << sumNvFinal << "   . Relative difference = " << abs(NvTfinal - sumNvFinal) / NvTfinal * 100.0 << " %" << endl;
    
    // Ceck total mass evolution
    cout << "integral mass [g/cm3]. Initial = " << mass0 << "   . Final = " << massFinal << endl;
    
    // CPU time
    cpuTime = clock() - cpuTime; // CPU time in [clicks]
    double cpuTimeSeconds = double(cpuTime) / CLOCKS_PER_SEC; //  CPU time in [s]
    cout << endl << "CPU time in seconds = " << cpuTimeSeconds << endl << endl;
    writeCPUtime(cpuTimeSeconds, pathProject, pathTargetGlobal);
    
    
}


void mainHomogeneousGasPhaseStandard(string pathProject, string pathTagetInputsHomogeneousPhase, string HomogeneousPhaseDataFileName, string HomogeneousPhaseLabelFileName, string gridType, int nBins, double mSupMax, double factorFirstCell, double mMin, double c2coef, double n0, double x0, int npTot, string initType, double smallestDetectedDiameter, string solverType, string aggloType, double kf, double primaryDiameter, double fractalDim, string surfType, string massTransferType, string nuclType, string PAHType, string A4correctionType, string growthType, string HACAType, int itTotal, double timePerIt, string pathTargetPsd, string pathTargetGlobal, double alphaHaca, string tStepType, double alphaAgglo, double maxGcoef, string sectionalMethod, double fs, string growthtStepType, double tFinal, int partPerCluster, int writeFreq, int hybridTrigger, int showPsdFreq, bool subItWrite, double subItWrite_tmin, double subItWrite_tmax, string pbeMethod, double firstCellmAvg, double gridIncrement, double alphaGrid, string initShape, double initNvTbin, int binInit)
{
    // CPU time
    clock_t cpuTime;
    cpuTime = clock();
    
    HomogeneousGasPhase gasPhase = HomogeneousGasPhase(pathProject, pathTagetInputsHomogeneousPhase, HomogeneousPhaseDataFileName, HomogeneousPhaseLabelFileName);
    
    //cout << gasPhase.getT(0.0) << endl;
    
    
    Grid grid1 = Grid();  // grid with no parameters
    if(gridType == "standard")
    {
        grid1 = Grid(gridType, nBins, mSupMax, factorFirstCell, mMin, c2coef); // for standard
    }
    else if(gridType == "geometricFs")
    {
        grid1 = Grid(nBins, firstCellmAvg, fs, gridType);
    }
    else if(gridType == "regular")
    {
        grid1 = Grid(nBins, firstCellmAvg, fs, gridType, gridIncrement);
    }
    else if(gridType == "exponential")
    {
        grid1 = Grid(nBins, firstCellmAvg, fs, gridType, gridIncrement, alphaGrid, mMin);
    }
    else
    {
        cout << "ERROR gridType initialization" << endl;
    }
    
    Grid* gridPtr = &grid1;
    grid1.showGrid();
    
    gridPtr->showGrid();
    
    

    AerosolPhase aerosolPhase1 = AerosolPhase();  // AerosolPhase with no parameters
    if(initShape == "exponential")
    {
        aerosolPhase1 = AerosolPhase(gridPtr, n0, x0, npTot, initType);
    }
    else if(initShape == "binInit")
    {
        aerosolPhase1 = AerosolPhase(gridPtr, npTot, binInit, initNvTbin);
    }
    else
    {
        cout << "ERROR initShape AerosolPhase initialization" << endl;
    }
    AerosolPhase* aerosolPtr = &aerosolPhase1;
    
    
    
    

    Psd psd1 = Psd(grid1, aerosolPtr, initNvTbin); // Psd with no parameters
    if(pbeMethod == "npBasic" || pbeMethod == "npBasicPart")
    {
        double initialNvT = aerosolPhase1.getInitNvT();
        psd1 = Psd(grid1, aerosolPtr, initialNvT, smallestDetectedDiameter);
        psd1.countAndSetAllNp();
        psd1.calcAndSetAllNv();
        
        psd1.setmAvgVar_as_mAvgVarStoch();
        psd1.setmAvgVarR_as_mAvgVarStoch();
    }
    else if(pbeMethod == "sectional" && initShape == "exponential")
    {
        //psd1 = Psd(grid1, n0, x0, "dm", "sectional");
        psd1 = Psd(grid1, n0, x0, "dm", "sectional", smallestDetectedDiameter);
        
    }
    else if(pbeMethod == "sectional" && initShape == "binInit")
    {
        double initialNvT = aerosolPhase1.getInitNvT();
        psd1 = Psd(grid1, binInit, initialNvT, "sectional", smallestDetectedDiameter);
    }
    else if( (pbeMethod == "npResidual" || pbeMethod == "npResidualPart" || pbeMethod == "hybrid" || pbeMethod == "hybridPart") && initShape == "exponential")
    {
        //psd1 = Psd(grid1, aerosolPtr, n0, x0, "dm", pbeMethod);
        psd1 = Psd(grid1, aerosolPtr, n0, x0, "dm", pbeMethod, smallestDetectedDiameter);
    }
    else if( (pbeMethod == "npResidual" || pbeMethod == "npResidualPart" || pbeMethod == "hybrid" || pbeMethod == "hybridPart") && initShape == "binInit")
    {
        double initialNvT = aerosolPhase1.getInitNvT();
        psd1 = Psd(grid1, aerosolPtr, initialNvT, smallestDetectedDiameter);
        psd1.countAndSetAllNp();
        psd1.calcAndSetAllNv();
        
        psd1.setmAvgVar_as_mAvgVarStoch();
        psd1.setmAvgVarR_as_mAvgVarStoch();
    }
    else
    {
        cout << "ERROR pbeMethod for Psd initialization" << endl;
    }
    
    // in order to have initial mAvgVar values. Case npBasic or npBasicPart treated above. Case npResidual, npResidualPart, hybrid or hybridPart treated directly in Psd creator.
    if(pbeMethod == "sectional")
    {
        psd1.setmAvgVar_as_mAvgConst();
        psd1.setmAvgVarR_as_mAvgConst();
    }
    
    Psd* psdPtr = &psd1;
    psdPtr->showPsd();
    
    
    
    AgglomerationTensor aggloTensor = AgglomerationTensor(psdPtr);
    AgglomerationTensor* aggloTensorPtr = &aggloTensor;
    
    
    
    HomogeneousGasPhase* gasPtr = &gasPhase;
    
    
    
    SootSourceTerm source = SootSourceTerm(gasPtr, psdPtr, solverType);
    SootSourceTerm* sourcePtr = &source;
    
    source.setAggloTensor(aggloTensorPtr);
    source.setAggloType(aggloType);
    
    source.setKf(kf);
    source.setImposedDpSource(primaryDiameter);
    source.setDf(fractalDim);
    
    source.setSurfType(surfType);
    
    source.setMassTransferType(massTransferType);
    
    source.setNuclType(nuclType);
    
    //source.setgammaNucl(gammaNucl0);
    
    source.setPAHType(PAHType);
    source.setA4correctionType(A4correctionType);
    
    source.setGrowthType(growthType);
    
    source.setHacaType(HACAType);
    
    source.setalphaHaca(alphaHaca);
    
    
    GrowthPark growthPark1 = GrowthPark(psdPtr, sourcePtr,  solverType, fs);
    GrowthPark* growthParkPtr = &growthPark1;
    
    //cout << "dotNv(i=1) growth park = " << growthParkPtr->dotNvi(1, 0.0) << endl;
    
    
    TimeStep tstep = TimeStep(alphaAgglo, sourcePtr, psdPtr, tStepType, maxGcoef, growthtStepType, sectionalMethod, growthParkPtr);
    TimeStep* timeStepPtr = &tstep;
    
    //cout << "growth target time step = " << timeStepPtr->growthTargetTimeStep(0.0) << endl;
    
    
    NpSolver solver = NpSolver(sourcePtr, aerosolPtr, psdPtr, itTotal, timePerIt, pathProject, pathTargetPsd, pathTargetGlobal, solverType, timeStepPtr, tFinal, pbeMethod, partPerCluster, writeFreq, sectionalMethod, growthParkPtr, hybridTrigger, showPsdFreq, subItWrite, subItWrite_tmin, subItWrite_tmax);
    
     
    // solve
    double mass0 = psd1.getmTot();
    
    solver.solve();
    
    double massFinal = psd1.getmTot();
    double NvTfinal = psd1.getNvT();
    double sumNvFinal = psd1.getSumNv();
    
    // show grid for verification
    grid1.showGrid();
    
    // Check sumNv vs NvT (number conservation test)
    cout << endl << "NvT = " << NvTfinal << "   . SumNv = " << sumNvFinal << "   . Relative difference = " << abs(NvTfinal - sumNvFinal) / NvTfinal * 100.0 << " %" << endl;
    
    // Ceck total mass evolution
    cout << "integral mass [g/cm3]. Initial = " << mass0 << "   . Final = " << massFinal << endl;
    
    // CPU time
    cpuTime = clock() - cpuTime; // CPU time in [clicks]
    double cpuTimeSeconds = double(cpuTime) / CLOCKS_PER_SEC; //  CPU time in [s]
    cout << endl << "CPU time in seconds = " << cpuTimeSeconds << endl << endl;
    writeCPUtime(cpuTimeSeconds, pathProject, pathTargetGlobal);
    
}


// older version with less parameters
void mainHomogeneousGasPhaseStandard(string pathProject, string pathTagetInputsHomogeneousPhase, string HomogeneousPhaseDataFileName, string HomogeneousPhaseLabelFileName, string gridType, int nBins, double mSupMax, double factorFirstCell, double mMin, double c2coef, double n0, double x0, int npTot, string initType, double smallestDetectedDiameter, string solverType, string aggloType, double kf, double primaryDiameter, double fractalDim, string surfType, string massTransferType, string nuclType, string PAHType, string A4correctionType, string growthType, string HACAType, int itTotal, double timePerIt, string pathTargetPsd, string pathTargetGlobal, double alphaHaca)
{
    HomogeneousGasPhase gasPhase = HomogeneousGasPhase(pathProject, pathTagetInputsHomogeneousPhase, HomogeneousPhaseDataFileName, HomogeneousPhaseLabelFileName);
    
    cout << gasPhase.getT(0.0) << endl;
    
    
    
    Grid grid1 = Grid(gridType, nBins, mSupMax, factorFirstCell, mMin, c2coef);
    Grid* gridPtr = &grid1;
    
    //gridPtr->showGrid();
    
    AerosolPhase aerosolPhase1 = AerosolPhase(gridPtr, n0, x0, npTot, initType);
    AerosolPhase* aerosolPtr = &aerosolPhase1;
    
    
    double initialNvT = aerosolPhase1.getInitNvT();
    
    Psd psd1 = Psd(grid1, aerosolPtr, initialNvT, smallestDetectedDiameter);
    psd1.countAndSetAllNp();
    psd1.calcAndSetAllNv();
    Psd* psdPtr = &psd1;
    
    //psdPtr->showPsd();
    
    AgglomerationTensor aggloTensor = AgglomerationTensor(psdPtr);
    AgglomerationTensor* aggloTensorPtr = &aggloTensor;
    
    
    HomogeneousGasPhase* gasPtr = &gasPhase;
    
    SootSourceTerm source = SootSourceTerm(gasPtr, psdPtr, solverType);
    SootSourceTerm* sourcePtr = &source;
    
    source.setAggloTensor(aggloTensorPtr);
    source.setAggloType(aggloType);
    
    source.setKf(kf);
    source.setImposedDpSource(primaryDiameter);
    source.setDf(fractalDim);
    
    source.setSurfType(surfType);
    
    source.setMassTransferType(massTransferType);
    
    source.setNuclType(nuclType);
    
    //source.setgammaNucl(gammaNucl0);
    
    source.setPAHType(PAHType);
    source.setA4correctionType(A4correctionType);
    
    source.setGrowthType(growthType);
    
    source.setHacaType(HACAType);
    
    source.setalphaHaca(alphaHaca);
    
    
    NpSolver solver = NpSolver(sourcePtr, aerosolPtr, psdPtr, itTotal, timePerIt,pathProject, pathTargetPsd, pathTargetGlobal, solverType);
    
    // solve
    solver.solve();
    
}

void writeCPUtime(double CPUtime, std::string pathProject, std::string pathTargetGlobal)
{
    string finalPath = pathProject.append(pathTargetGlobal);
    string dat = "CPUtime.dat";
    finalPath = finalPath.append(dat);
    
    ofstream stream1(finalPath.c_str());
    if(stream1) // error test
    {
        stream1 << "#CPU time in [s]" <<endl;
        stream1 << CPUtime << endl;
    }
    else
    {
        cout << "ERROR: Impossible to open the file." << endl;
    }
}
