//
//  tools.cpp
//  soot_stochastic
//
//  Created by Alexandre Bouaniche on 22/06/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#include "tools.hpp"

#include <iostream>
#include <string>
#include <vector>
#include <cmath>

// for random function
#include <cstdio>
#include <cstdlib>
#include <ctime>


using namespace std;

/*
 Picks random integer values. Returns a vector of size nbPicked of random integers with values between 0 and maxVal INCLUDED.
 
 DEPENDS ON t!! For the same t, picks the same list. (Random within the list but two identical lists if calculated during the same run without dependency on t)
 
 nbPicked must be an EVEN number for mixing
 
 */
vector<int> randomList(double t, int nbPicked, int maxVal)
{
    int random_integer = 0;
    vector<int> randomList;
    srand(time(NULL)+t); // initialisation de rand
    int i;
    for(i=0; i<nbPicked; i++)
    {
        random_integer = rand()%(maxVal+1);
        randomList.push_back(random_integer);
    }
    return randomList;
}


vector<int> randomListWithoutDuplicate(double t, int nbPicked, int maxVal)
{
    int random_integer = 0;
    vector<int> randomList;
    srand(time(NULL)+t); // initialisation de rand
    int i;
    for(i=0; i<nbPicked; i++)
    {
        random_integer = rand()%(maxVal+1);
        
        while(isIntInlist(random_integer, randomList))
            random_integer = rand()%(maxVal+1);
        
        randomList.push_back(random_integer);
    }
    return randomList;
}



double maxColi(vector<vector<double> > const& matrix, int col)
{
    double max(0);
    max = matrix[0][col];
    int i(0);
    for(i=0; i<matrix.size(); i++)
    {
        if(max<matrix[i][col])
            max = matrix[i][col];
    }
    return max;
}


int rounding(double d)
{
    double dSuperior = d+0.5;
    int rounded = floor(dSuperior);
    return rounded;
}
 

bool isIntInlist(int value, vector<int> list)
{
    int i=0;
    bool is = 0;
    for(i=0; i< list.size(); i++)
    {
        if(list[i] == value)
            is = 1;
    }
    return is;
}


double frand_a_b(double a, double b)
{
    return ( rand()/(double)RAND_MAX ) * (b-a) + a;
}

double piecewise_linear_distrib_w(double wInf, double wSup, double bInf, double bSup){
    // see http://www.cplusplus.com/reference/random/piecewise_linear_distribution/
    // adding = before the { arrays
    
    // generate a random b value between bInf and bSup based on a linear probability density function defined by wInf and wSum, and bInf and bSup so that the PDF values at bInf and bSup are pInf and pSup.
    
    double b(0.0);
    
    unsigned seed = (int)std::chrono::system_clock::now().time_since_epoch().count();
    std::default_random_engine generator (seed);

    std::array<double,2> intervals = {bInf, bSup};
    std::array<double,2> weights = {wInf, wSup};
    std::piecewise_linear_distribution<double> distribution (intervals.begin(),intervals.end(),weights.begin());
    b = distribution(generator);
    
    return b;
    
}

double piecewise_linear_distrib_n(double mInf, double mSup, double nInf, double nSup){
    // see http://www.cplusplus.com/reference/random/piecewise_linear_distribution/
    // adding = before the { arrays
    
    // based on mInf and mSup: used for PSD initialization. For reallocation during calculation, could be necessary to make the linear distrib over [mAvgVar - deltam/2; mAvgVar + deltam/2
    // uses piecewise_linear_distrib(double wInf, double wSup, double bInf, double bSup). Applied to a problem with physical variables m, N and n
    
    double m(0.0);
    
    /*
    // integral of n over [mInf; mSup] is equal to Ni
    // integral of p (probability density function over [mInf; mSup] must be equal to 1
    double pInf = nInf / Ni;
    double pSup = nSup / Ni;
    
    // see calculus on paper
    //double wSup = (mSup - mInf) * (pSup - pInf) / 4.0;
    //double wInf = 1 - wSup;
    double wInf = 1.0; // choose wInf = 1 and calculate wSup as a function of wInf.
    double K = (pSup - pInf) * (mSup - mInf) / 2.0;
    double wSup = wInf * (1.0 + K) / (1.0 - K);
     */
    
    // should not use negative coeficients.
    if (nInf < 0.0)
    {
        nInf = 0.0;
    }
    if (nSup < 0.0)
    {
        nSup = 0.0;
    }
    
    double nAvg = (nInf + nSup) / 2.0;
    
    double wInf = 0.0;
    double wSup =  0.0;
    if (nAvg > 0.0)
    {
        wInf = nInf / nAvg;
        wSup = nSup / nAvg;
    }
    
    m = piecewise_linear_distrib_w(wInf, wSup, mInf, mSup);
    //cout << "m = " << m << "   wInf = " << wInf << "   wSup = " << wSup << "   mInf = " << mInf << "   mSup = " << mSup << endl;
    return m;
}
